;***************************
; Generate the transmission filters for the IMaX filterpolarimeter
;***************************
pro imax_gaussian

	nl = 200
	lambda0 = 5250.209d0
	lambda_left = 5249.8d0
	lambda_right = 5250.8d0
	dlambda = 0.5d0
	
	l = dindgen(nl) / (nl-1.d0) * (lambda_right - lambda_left) + lambda_left

	trans = dblarr(5, nl)

	lambda0_pos = lambda0 + [-80.d0, -40.d0, 40.d0, 80.d0, 240.d0] * 1.d-3

; A Gaussian with 85 mA mimicks the effect of stray-light according to Valentin
	width = 85.d0 * 1.d-3

	cwpal
	plot, [0,0], xran=[lambda_left, lambda_right], xsty=1, yran=[0,1], ysty=1
	for i = 0, 4 do begin
		prof = exp(-(l-lambda0_pos[i])^2 / width^2)
		normaliz = tsum(l, prof)
		
		oplot, l, prof, col=i+1
		
		trans[i,*] = prof / normaliz
	endfor

	openw,2,'imax.filters',width=132
	printf,2,'IMaX filter position and transmission'
	printf,2,'N. filters, n. wavelength points'
	printf,2,5,nl
	printf,2,'Positions'
	printf,2,lambda0_pos
	printf,2,'Transmission'
	for i = 0, nl-1 do begin
		printf,2,l[i], trans[*,i]
	endfor
	close,2
	stop
end


;***************************
; Generate the transmission filters for the IMaX filterpolarimeter
; The PSF is not complete
;***************************
pro imax_etalon

	nl = 100
	lambda0 = 5250.209d0
	lambda_left = 5249.8d0
	lambda_right = 5250.8d0
	dlambda = 0.5d0
	
	l = dindgen(nl) / (nl-1.d0) * (lambda_right - lambda_left) + lambda_left

	trans = dblarr(5, nl)

	lambda0_pos = lambda0 + [-80.d0, -40.d0, 40.d0, 80.d0, 240.d0] * 1.d-3

	psf = ddread('psf_imax.dat')

	cwpal
	plot, [0,0], xran=[lambda_left, lambda_right], xsty=1, yran=[0,1], ysty=1
	for i = 0, 4 do begin
		prof = interpol(psf[1,*], psf[0,*]*1.d-3, (l-lambda0_pos[i]))
		
		normaliz = tsum(l, prof)
		
		oplot, l, prof, col=i+1
		
		trans[i,*] = prof / normaliz
	endfor

	openw,2,'imax.filters',width=132
	printf,2,'IMaX filter position and transmission'
	printf,2,'N. filters, n. wavelength points'
	printf,2,5,nl
	printf,2,'Positions'
	printf,2,lambda0_pos
	printf,2,'Transmission'
	for i = 0, nl-1 do begin
		printf,2,l[i], trans[*,i]
	endfor
	close,2
	stop
end
