Module NLTE_vars
  Use Model_structure
!
! Structure containing physical variables needed for the NLTE computation
!
  Type NLTE_variables
!
! NDEP is the number of depth-points in the NLTE grid
! Nstar(NK,NDEP) and N(NK,NDEP) are LTE and NLTE population number 
! densities in cm^-3
!
! W, F and C are the rate matrices (NK,NK,NDEP): F->Fixed radiative trans,
!            C->Collisional rates. 
!
! BPlanck: Planck function B_nu
! Xnorm(1:NDEP)=Background continuum opacity (g/cm^2)
! XMu(1:NMU), WMU(1:NMU)=Gaussian integration weights
!
! Wphi=1./Integral_{angle, freq} (Phi_{mu,nu}) d_mu d_nu (see voigprofs)
! Sl(1:NDEP,1:NLIN)=Line source function (cgs)
! Source_f(1:NDEP, 1:NQ, 1:NLIN+NCNT)=Total source function (cgs)
!
! Error is set to .true. if something has gone wrong in the NLTE calculation
!  (e.g., non-convergence, NaN results, negative populations or other 
!  oddities) 
     Integer :: NDEP
     Real, Dimension(:,:), pointer :: Nstar, N, BPlanck, WPhi, Sl
     Real, Dimension(:,:,:), pointer :: W, F, C, Source_f
     Real, Dimension(:), pointer :: XMu, WMu, Xnorm
     Logical :: Error
     Type (Model) :: Atmo
!
  End Type NLTE_variables

End Module NLTE_vars
