! This routine computes quadrature points and weights for line and
! continuum transitions. In the case of continuum transitions, Atom%AlphaC
! is also filled assuming a nu^-3 dependence (unless Q0 or QMAX .lt. 0, in 
! which case it is explicitly given in the file ATOM).
!
! For cont transitions, QMAX is the shortest wavelength considered (in Angs)
! For line transitions, QMAX is the farthest wavelength (in Doppler units)
!
Subroutine FreqQuad(Atom, NLTEInput)
  Use Param_structure
  Use NLTE_Atom_structure
  Use NLTE_inp
  Implicit None
  Type (NLTE_Atom) :: Atom
  Type (NLTE_input) :: NLTEInput
  Real, Dimension (0:MaxNFreqs) :: X
  Integer :: itran, ix, ncore, nwing
  Real :: DX
!
! Line transitions
!
  Do itran=1, Atom%NLIN
     ncore=(Atom%NQ(itran)-1)/2 ! One point is for line center (Q=0)
     nwing=(Atom%NQ(itran)-1)-ncore
!
! Symmetric profile, consider only 1 side
!
! core
     Atom%Q(1,itran)=0. ! line center
     DX=Atom%Q0(itran)/(ncore)
     Do ix=1, ncore
        Atom%Q(ix+1,itran)=ix*DX
     End do
! wing
     DX=( Log10(Atom%QMAX(itran))-Log10(Atom%Q0(itran)) )/ & 
          (nwing)
     Do ix=1, nwing
        Atom%Q(1+ncore+ix,itran)=10.**(Log10(Atom%Q0(itran))+(ix)*DX)
     End do
!
! Asymmetric profile, consider both sides
!
     If (.not. NLTEInput%VelFree) then
        X(1:Atom%NQ(itran))=Atom%Q(1:Atom%NQ(itran),itran)
        Do ix=1, Atom%NQ(itran)/2
           Atom%Q(ix,itran)=-X(Atom%NQ(itran)+2-ix*2)
           Atom%Q(Atom%NQ(itran)/2+ix,itran)=X(ix*2)
        End do
     End if
!
! Weights
!
     Atom%WQ(1,itran)=0.5*(Atom%Q(2,itran)-Atom%Q(1,itran))
     Atom%WQ(Atom%NQ(itran),itran)=0.5*(Atom%Q(Atom%NQ(itran),itran)- &
          Atom%Q(Atom%NQ(itran)-1,itran))
     Do ix=2, Atom%NQ(itran)-1
        Atom%WQ(ix,itran)=0.5*(Atom%Q(ix+1,itran)-Atom%Q(ix-1,itran))
     End do
     If (NLTEInput%VelFree) Atom%WQ(2:,itran)=2.*Atom%WQ(2:,itran)
  End do
!
! Continuum transitions
!
  Do itran=Atom%NLIN+1, Atom%NLIN+Atom%NCNT
     X(0)=cc/Atom%Alamb(itran)*1.e8 ! Freq in Hz
     If (atom%QMAX(itran) .ge. 0.) then ! Compute frequency grid
        X(Atom%NQ(itran))=cc/Atom%QMAX(itran)*1.e8 ! Hz
        DX=(X(Atom%NQ(itran)) - X(0))/Atom%NQ(itran)
        Do ix=1, Atom%NQ(itran)
           X(ix)=X(ix-1)+DX
        End do
        Do ix=1, Atom%NQ(itran)
           Atom%Q(ix,itran)=cc*1e-5/NLTEInput%QNORM*(X(ix)/X(0)-1.)
           Atom%AlphaC(ix,itran)=Atom%f(itran)*((X(0)/X(ix))**3)
        End do
        Atom%FRQ(0:Atom%NQ(itran),itran)=X(0:Atom%NQ(itran))
     Else ! Grid was read from ATOM file
        Atom%FRQ(1:Atom%NQ(itran),itran)=cc/Atom%Q(1:Atom%NQ(itran),itran)*1.e8
        Atom%Q(1:Atom%NQ(itran),itran)=cc*1.e-5/NlteInput%QNORM* &
             (Atom%FRQ(1:Atom%NQ(itran),itran)/Atom%FRQ(0,itran)-1.)
     End if
     Atom%WQ(1,itran)=0.5*(Atom%Q(2,itran)-Atom%Q(1,itran))
     Atom%WQ(Atom%NQ(itran),itran)=0.5*(Atom%Q(Atom%NQ(itran),itran)- &
          Atom%Q(Atom%NQ(itran)-1,itran))
     Do ix=2, Atom%NQ(itran)-1
        Atom%WQ(ix,itran)=0.5*(Atom%Q(ix+1,itran)-Atom%Q(ix-1,itran))
     End do
  End do
!

End Subroutine FreqQuad
