! This subroutine computes the Zeeman-split profiles
!
Subroutine Profiles(Params, Line, nwlengths, nw, IndexWave, Wave, Atmo, Damp, Dldop, &
             Line_op, Phi_I, Phi_Q, Phi_U, Phi_V, Psi_Q, Psi_U, Psi_V, &
             Cont_op_5000, Incomplete)
  Use Param_structure
  Use Model_structure
  Use Line_data_structure
  Implicit None
  Type (Parameters) :: Params
  Type (Line_data) :: Line
  Type (Model) :: Atmo
  Real, Dimension(nwlengths) :: Wave, DWave
  Real, Dimension(nw) :: vv
  Integer, Dimension(nwlengths) :: IndexWave
  Integer :: ind, npoints, nr, nl, np, nwlengths, nw
  Integer, Dimension(2) :: ji, jf, mult
  Character, Dimension(2) :: desig
  Real, Dimension (transitions) :: dlp, dll, dlr, sp, sl, sr
  Real, Dimension (Params%n_points) :: sg, cg, s2g, c2g, sf, cf
  Real, Dimension (Params%n_points) :: Damp, Dldop, Line_op, Cont_op_5000
  Real, Dimension (nwlengths, Params%n_points) :: Phi_I, Phi_Q, Phi_U, Phi_V, &
       Psi_Q, Psi_U, Psi_V
  Real :: t13, t14, dldopcm, dldopcms, a, v, mag
  Real*8 :: etar, vetar, getar, ettar, ettvr, esar, vesar, gesar
  Real*8 :: essar, essvr, ettmr, essmr
  Real*8 :: etal, vetal, getal, ettal, ettvl, esal, vesal, gesal
  Real*8 :: essal, essvl, ettml, essml
  Real*8 :: etap, vetap, getap, ettap, ettvp, esap, vesap, gesap
  Real*8 :: essap, essvp, ettmp, essmp
  Real*8 :: tm, tn, sm, sn, tpm, spm
  Real*8, Dimension(nwlengths) :: vtm, vtn, vsm, vsn, vtpm, vspm
  Real*8, Dimension(nwlengths) :: vec_etar, vec_esar, vec_etal, &
       vec_esal, vec_etap, vec_esap
  Real :: dlo, wc
  Real, Dimension (2) :: tam
  Integer :: iwave
  Logical :: Incomplete, Exists
  Logical, Save :: FirstTime
! For Hyperfine Structure case
  Integer, Save :: Unit, nHFlines, ilin
  Real :: MagField
  Logical, Save :: Constant_B
  Integer, Dimension (:), Allocatable, Save :: HFIndex
  Real, Dimension(:), Allocatable, Save :: Alow, Blow, Aup, Bup, SpinI
! Special case definitions for 15652
  Real, Dimension (9) :: dlp15652, dll15652, dlr15652, &
       sp15652, sl15652 ,sr15652
  Data dlp15652/-4.4000149E-2,-3.3000112E-02,-2.2000074E-02,-1.1000037E-02, &
       0.00,1.1000037E-02,2.2000074E-02,3.3000112E-02,4.4000149E-02/
  Data dll15652/1.466000,1.477000,1.488000,1.499000, &
    1.510000,1.521000,1.532000,1.543000,1.554000/
  Data dlr15652/-1.554000,-1.543000,-1.532000,-1.521000, &
   -1.510000,-1.499000,-1.488000,-1.477000,-1.466000/
  Data sp15652/5.4545455E-02,9.6969694E-02,0.1272727,0.1454545, &
   0.1515152,0.1454545,0.1272727,9.6969694E-02,5.4545455E-02/
  Data sl15652/6.0606059E-03,1.8181818E-02,3.6363635E-02,6.0606062E-02, &
   9.0909094E-02,0.1272727,0.1696970,0.2181818,0.2727273/
  Data sr15652/0.2727273,0.2181818,0.1696970,0.1272727, &
   9.0909094E-02, 6.0606062E-02, 3.6363635E-02, 1.8181818E-02, 6.0606059E-03/
!
  Data FirstTime/.TRUE./
! If first time, check for hyperfine data file
  If (FirstTime) then
     Inquire(File='hyperfine.dat', Exist=Exists)
     nHFlines=0
     If (Exists) then ! Read HFS data and initialize routine
        If (Params%Printout .ge. 1) Print *,'Reading hyperfine.dat file'
        Call Open_file(Unit,'hyperfine.dat')
        Read (Unit,*) nHFlines
        Allocate (HFIndex(nHFlines))
        Allocate (Alow(nHFlines))
        Allocate (Blow(nHFlines))
        Allocate (Aup(nHFlines))
        Allocate (Bup(nHFlines))
        Allocate (SpinI(nHFlines))
        Do ilin=1, nHFlines
           Read(Unit,*) HFIndex(ilin),Alow(ilin),Blow(ilin),Aup(ilin), &
                Bup(ilin),SpinI(ilin)
        End do
        Close (Unit)
        Constant_B=.FALSE.
        If (MaxVal(Atmo%B_str)-MinVal(Atmo%B_str) .lt. 10) Constant_B=.TRUE.
        Call fctsg ! Initialize
     End if
     FirstTime=.FALSE.
  End if
!
  Incomplete=.FALSE.
  Phi_I(:,:)=0.
  Phi_Q(:,:)=0.
  Phi_U(:,:)=0.
  Phi_V(:,:)=0.
  Psi_Q(:,:)=0.
  Psi_U(:,:)=0.
  Psi_V(:,:)=0.
  t13=0. ! Derivative of log (Dldop) with temperature
  t14=0. ! Derivative of log (Dldop) with microturbulence
  npoints=Params%n_points
  ji(1)=int(Line%J_low)
  ji(2)=int(Line%J_up)
  jf(1)=int(10*(Line%J_low-ji(1)))
  jf(2)=int(10*(Line%J_up-ji(2)))
  tam(1)=Line%J_low
  tam(2)=Line%J_up
  mult(1)=Line%Mult_low
  mult(2)=Line%Mult_up
  desig(1)=Line%Desig_low
  desig(2)=Line%Desig_up
  dlo=4.6686E-5*(Line%Wlength**2.)*1.E-16 ! Lambda_B in mA
! Special case for FeI 15652 A ?
  If (Line%Wlength .ge. 15652.7 .and. Line%Wlength .le. 15652.9 &
       .and. Line%Atomic_number .eq. 26 .and. Line%Ion_stage .eq. 1) then 
     If (FirstTime .and. Params%Printout .ge. 1) &
          Print *,'Special case for FeI 15652'
     np=9
     nl=9
     nr=9
     dlp(1:np)=dlp15652(1:np)*dlo
     dll(1:nl)=dll15652(1:nl)*dlo
     dlr(1:nr)=dlr15652(1:nr)*dlo
     sp(1:np)=sp15652(1:np)
     sl(1:nl)=sl15652(1:nl)
     sr(1:nr)=sr15652(1:nr)
  Else
     Call Zeeman(transitions, mult, desig, tam, ji, jf, dlo, np, nl, nr, &
          dlp, dll, dlr, sp, sl, sr)
     If (np .gt. transitions .or. nl .gt. transitions .or. &
          nr .gt. transitions) then
        Print *,'Too many transitions (',np,',', nl,',', nr,')'
        Print *,'Profiles has been prepared to work with ',transitions, &
             '. STOP in Profile'
        Stop
     End if
  End if
  sg(1:npoints)=Sin(Atmo%b_inc(1:npoints)*pi/180.)
  cg(1:npoints)=Cos(Atmo%b_inc(1:npoints)*pi/180.)
  s2g(1:npoints)=sg(1:npoints)*sg(1:npoints)
  c2g(1:npoints)=1.+cg(1:npoints)*cg(1:npoints)
  sf(1:npoints)=Sin(2.*Atmo%b_azi(1:npoints)*pi/180.)
  cf(1:npoints)=Cos(2.*Atmo%b_azi(1:npoints)*pi/180.)
  wc=Line%Wlength*1.E-8/cc ! Inverse of line frequency (in s)
  Do iwave=1, nwlengths
     DWave(iwave)=Wave(iwave)-Line%Wlength
  End do
  Do ind=npoints, 1, -1
     a=Damp(ind)
     dldopcm=Dldop(ind)*1.E-8 ! Dldop in cm
     dldopcms=Dldop(ind)/Line%Wlength*cc ! Dldop in cm/s
     vv(1:nw)=DWave(IndexWave(1:nw))/Dldop(ind) - Atmo%v_los(ind)/dldopcms
     mag=Atmo%B_str(ind)/dldopcm
     If (nHFlines .gt. 0) then
        Do ilin=1, nHFlines
           If (Line%index .eq. HFIndex(ilin)) then 
              If (.not. Constant_B .or. ind .eq. npoints) then
                 Call HFInterface(Line, Atmo%B_str(ind), np, nl, nr, &
                      dlp, dll, dlr, sp, sl, sr, Alow(ilin), Blow(ilin), &
                      Aup(ilin), Bup(ilin), SpinI(ilin))
              End if
              mag=Max(Atmo%B_str(ind),1.e-2)/dldopcm ! Make sure B .ne. 0
           End if
        End do
     End if

     If (Params%speed .eq. 0) then ! Use old point-by-point voigt
        Do iwave=1, nw
           v=vv(iwave)

           call mvoigt(nr,dlr,sr,a,v,mag,t13,t14,wc,dldopcm,etar,vetar, &
                getar,ettar,ettvr,esar,vesar,gesar,essar,essvr, &
                ettmr,essmr)
           call mvoigt(nl,dll,sl,a,v,mag,t13,t14,wc,dldopcm,etal,vetal, &
                getal,ettal,ettvl,esal,vesal,gesal,essal,essvl, &
                ettml,essml)
           call mvoigt(np,dlp,sp,a,v,mag,t13,t14,wc,dldopcm,etap,vetap, &
                getap,ettap,ettvp,esap,vesap,gesap,essap,essvp, &
                ettmp,essmp)
           tm=.5*(etar+etal)
           tn=.5*(etar-etal)
           sm=.5*(esar+esal)
           sn=.5*(esar-esal)
           tpm=.5*(etap-tm)
           spm=.5*(esap-sm)
           Phi_I(IndexWave(iwave),ind)=Line_op(ind)*0.5*(etap*s2g(ind)+tm*c2g(ind))
           Phi_Q(IndexWave(iwave),ind)=Line_op(ind)*tpm*s2g(ind)*cf(ind)
           Phi_U(IndexWave(iwave),ind)=Line_op(ind)*tpm*s2g(ind)*sf(ind)
           Phi_V(IndexWave(iwave),ind)=Line_op(ind)*tn*cg(ind)
           Psi_Q(IndexWave(iwave),ind)=Line_op(ind)*spm*s2g(ind)*cf(ind)
           Psi_U(IndexWave(iwave),ind)=Line_op(ind)*spm*s2g(ind)*sf(ind)
           Psi_V(IndexWave(iwave),ind)=Line_op(ind)*sn*cg(ind)
        End do
     Else ! Use new vector voigt function
        Call mvoigt2(nr,nw,dlr,sr,a,vv,mag,wc,dldopcm,vec_etar,vec_esar)
        Call mvoigt2(nl,nw,dll,sl,a,vv,mag,wc,dldopcm,vec_etal,vec_esal)
        Call mvoigt2(np,nw,dlp,sp,a,vv,mag,wc,dldopcm,vec_etap,vec_esap)
        vtm(1:nw)=.5*(vec_etar(1:nw)+vec_etal(1:nw))
        vtn(1:nw)=.5*(vec_etar(1:nw)-vec_etal(1:nw))
        vsm(1:nw)=.5*(vec_esar(1:nw)+vec_esal(1:nw))
        vsn(1:nw)=.5*(vec_esar(1:nw)-vec_esal(1:nw))
        vtpm(1:nw)=.5*(vec_etap(1:nw)-vtm(1:nw))
        vspm(1:nw)=.5*(vec_esap(1:nw)-vsm(1:nw))
        Phi_I(IndexWave(1:nw),ind)=Line_op(ind)*0.5*(vec_etap(1:nw)*s2g(ind)+vtm(1:nw)*c2g(ind))
        Phi_Q(IndexWave(1:nw),ind)=Line_op(ind)*vtpm(1:nw)*s2g(ind)*cf(ind)
        Phi_U(IndexWave(1:nw),ind)=Line_op(ind)*vtpm(1:nw)*s2g(ind)*sf(ind)
        Phi_V(IndexWave(1:nw),ind)=Line_op(ind)*vtn(1:nw)*cg(ind)
        Psi_Q(IndexWave(1:nw),ind)=Line_op(ind)*vspm(1:nw)*s2g(ind)*cf(ind)
        Psi_U(IndexWave(1:nw),ind)=Line_op(ind)*vspm(1:nw)*s2g(ind)*sf(ind)
        Psi_V(IndexWave(1:nw),ind)=Line_op(ind)*vsn(1:nw)*cg(ind)
     End if
     If (Phi_I(1, ind) .gt. 1.e-2*Cont_op_5000(ind)) &
          Incomplete=.TRUE.
     If (Phi_I(nwlengths, ind) .gt. 1.e-2*Cont_op_5000(ind)) &
          Incomplete=.TRUE.
  End do
  Return
End Subroutine Profiles
