
      SUBROUTINE SOPAS(ID,IHTOT,FREQ,PEL,TEL,PG,CHIO,CHIE,ETA)
C
C********************************************************IAC 08.06.97
C
C This routine gives possibility to use the OPACITY package of
C         Nataliya G. Shchukina 
C e-mail:
C         shchukin@mao.kiev.ua (Kiev Ukraine)
C         
C         natasha@ll.iac.es
C
C****************************************************************                                                     
C INPUT:
C-------
C        ID     is the depth point number 
C
C !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C   Pen^a, please use    ID=1 !!!
C                     IHTOT=2 !!!
C !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C       IHTOT   is the parameter used for the choice 
C                  of the type of the ionization equilibrium solution
C                
C        FREQ   is a  frequency in [second-1]
C         PEL   is an electron pressure
C         TEL   is an electron emperature
C          PG   is a  gas pressure
C
c      There is a parameter called IFUDGE passed to chigm in a common, 
c      in the form
c               COMMON/FUDGE/IFUDGE      
c      IFUDGE=1 -> Consider fudge opacities
c      IFUDGE=0 -> Do not consider fudge opacities
c
c      Hector: I have hardwired this value to 1 in  chigm.f
C NB !!!  
C           You need real value PG only if IHTOT > 0
C                                       if IHTOT < 0 you can put PG=0. 
C
C OUTPUT: background opacity
C--------
C       CHIO   is   the continuum absorption coefficient due to
C
C                 - hydrogen negative H- (BF+FF);
C                 - hydrogen neutral  H  (BF+FF);
C                 - molecule H2+
C
C                 - metals (BF):
C                   SI - BF from first 8 levels,
C                   C  -               8 levels,
C                   MG -               8 levels,
C                   AL -               8 levels,
C                   Fe                 2 levels.
C   
C       CHIE   is the scattering coefficient for free electrons and 
C                                                neutral hydrogen
C
C         ETA  is  the continuum emission coefficient
C NB!
C         ETA    does NOT contain scattering part 
C
C C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C                          WARNING!
C
C  CHIG gives absorb. & emis. divided by density RHO!
C             If Z - scale   in [cm],
C                populations in [cm-3]
C                            then
C  CHIO, CHIE, ETA has to be in [cm-1]
C
C**********************************************************************
C
C  There are TWO choices for 
C  solution of ionization equilibrium equations:
C----------
C Choice 1:
C----------
C
C  IHTOT > 0 - density RHO and total number of hydrogen atoms HTOT
C              are known
C              from gas pressure PG and electron pressure PEL 
C             
C
C              subroutine STATP is called.
C
C  IHTOT = 1   the solution without molecules 
C          2                with    molecules H2+ and H2.
C
C----------
C Choice 2:
C----------
C
C   IHTOT < 0 - density RHO and total number of hydrogen atoms HTOT 
C               are determined with new
C               chemical composition.
C
C              subroutine STATM is called.
C
C  IHTOT =-1   the solution without molecules
C  IHTOT =-2                with    molecules H2+ and H2.
C
C

C   Contributitors in in the electron density NE: 
C   H, Si, C, Mg, Al, Fe, He, N, O, Ne, Na, P, S, Ar, Ca, Cr, Mn.
C

C********************************************************************
C
C  PARAMAX contains dimensions of arrays 
C
C                INCLUDE 'PARAMAX'
      PARAMETER (MDEP=1,MK=1,MFT=1)  
C                INCLUDE 'CCONST'
      COMMON/CCONST/ EE,HH,CC,BK,EM,UU,HCE,HC2,HCK,EK,PI
                
      LOGICAL PHOTO
      REAL NP,M,NATOM,NHYD,NE
      REAL HYDR(MDEP), GAS(MDEP)
      
      COMMON/ATWT/ ATW,SUM
C               
      COMMON/MODEL/ NE(MDEP),TEMP(MDEP),PE(MDEP),NATOM(MDEP)                
C
      COMMON/OPAC/ M(MDEP),RHO(MDEP),CHE(MFT),SIG(MFT),                    
     ,FRQ(MFT),WT(MFT),PHOTO(MFT),NXTION(MK),BNU(MFT)
C
      COMMON/POPUL/ HTOT(MDEP),NHYD(5,MDEP),NP(MDEP)
C

             PE(ID)=PEL
C
             NE(ID)=PEL/TEL/BK
           TEMP(ID)=TEL 
            GAS(ID)=PG  
            
         IF(IHTOT.GT.0) GO TO 1
         
         IF(IHTOT.LT.0) GO TO 2
         
          STOP
          
                
C***************************************************************
C 
C calculate HTOT if IHTOT > 0
C and
C call ionization equilibrium routine 
C       STATP (IHTOT > 0)
C       STATM (IHTOT < 0) 
C
    1  HYDR(ID)=(GAS(ID)-PE(ID))/TEMP(ID)/BK
       
         RHO(ID)=1.66E-24*HYDR(ID)*ATW
C         PRINT *,'sopas RHO=',RHO(ID)
         
         HYDR(ID)=HYDR(ID)/SUM 
            HTOT(ID)=HYDR(ID)
C            PRINT *,HTOT(ID) 
                   
          CALL STATP(ID,IHTOT)
                        
         GO TO 3
  
    2      CALL STATM(ID,IHTOT)     


C****************************************************************
C
C   call opacity routine CHIG 

    3     CONTINUE      

          CALL CHIG(ID,FREQ,CHIO,CHIE,ETA)
c          PRINT *,'absorb. cont. coef. CHIO = ',CHIO
c          PRINT *,'scat.   cont. coef. CHIE = ',CHIE         
c          PRINT *,'emis.   cont. coef. ETA  = ',ETA 
          
          SF=ETA/CHIO 
c          PRINT *,'ETA/CHIO = ',SF
          
c          PRINT *,'Planck =',PLANCK(FREQ,TEL)
          
                   
           RETURN
           END           
             
             
             
             
             
