! This subroutine solves the Stokes radiative transfer equation as 4 
! independent scalar equations, using the Weakly Polarizing Media
! approximation of Sanchez Almeida and Trujillo Bueno 1999, ApJ, in
! press.
!
Subroutine WPM(npoints, ltau_500, Ab, Source_f, Emergent_stokes)
  Implicit None
  Real, Dimension (npoints) :: ltau_500, Source_f, tau_nu
  Real, Dimension (npoints) :: Opac, tau_500, Stokes_component
  Real, Dimension (4, 4, npoints) :: Ab
  Real, Dimension (4) :: Emergent_stokes
  Integer :: npoints, ipoint
!
! First construct the tau_5000 depth scale
!
  tau_500(1:npoints)=10**ltau_500(1:npoints)
!
! Solve for Stokes I without polarization
!
  Opac(1:npoints)=Ab(1, 1, 1:npoints)
  tau_nu(1)=tau_500(1)*Opac(1)
  Do ipoint=2, npoints
     tau_nu(ipoint)=tau_nu(ipoint-1)+ &
          (tau_500(ipoint)-tau_500(ipoint-1))* &
          0.5*(Opac(ipoint)+Opac(ipoint-1))
  End do 
  Call Scalar_hermite(npoints, tau_nu, Source_f, Stokes_component)
  Emergent_stokes(1)=Stokes_component(1)
!
! Now solve for I + V and subtract the I computed above
!
  Opac(1:npoints)=Ab(1, 1, 1:npoints) + Ab(1, 4, 1:npoints)
  tau_nu(1)=tau_500(1)*Opac(1)
  Do ipoint=2, npoints
     tau_nu(ipoint)=tau_nu(ipoint-1)+ &
          (tau_500(ipoint)-tau_500(ipoint-1))* &
          0.5*(Opac(ipoint)+Opac(ipoint-1))
  End do 
  Call Scalar_hermite(npoints, tau_nu, Source_f, Stokes_component)
  Emergent_stokes(4)=Stokes_component(1) - Emergent_stokes(1) ! I+V - I
!
! Now solve for I + Q and subtract the I computed above
!
  Opac(1:npoints)=Ab(1, 1, 1:npoints) + Ab(1, 2, 1:npoints)
  tau_nu(1)=tau_500(1)*Opac(1)
  Do ipoint=2, npoints
     tau_nu(ipoint)=tau_nu(ipoint-1)+ &
          (tau_500(ipoint)-tau_500(ipoint-1))* &
          0.5*(Opac(ipoint)+Opac(ipoint-1))
  End do 
  Call Scalar_hermite(npoints, tau_nu, Source_f, Stokes_component)
  Emergent_stokes(2)=Stokes_component(1) - Emergent_stokes(1) ! I+Q - I
!
! Now solve for I + U and subtract the I computed above
!
  Opac(1:npoints)=Ab(1, 1, 1:npoints) + Ab(1, 3, 1:npoints)
  tau_nu(1)=tau_500(1)*Opac(1)
  Do ipoint=2, npoints
     tau_nu(ipoint)=tau_nu(ipoint-1)+ &
          (tau_500(ipoint)-tau_500(ipoint-1))* &
          0.5*(Opac(ipoint)+Opac(ipoint-1))
  End do 
  Call Scalar_hermite(npoints, tau_nu, Source_f, Stokes_component)
  Emergent_stokes(3)=Stokes_component(1) - Emergent_stokes(1) ! I+U - I
! Done!
  Return
End Subroutine WPM
