! This subroutine reads the atomic file data from the ATOM file
! The format is similar to that of MULTI, except that comments
! are marked by the ! symbol, instead of the *
! There are some limitations as to the features supported here
! Check forward/NLTE/NLTE_atom_struct.f90 for details on these 
! limitations.
!
Subroutine Read_atom(Atom)
  Use NLTE_Atom_structure
  Use Param_structure
  Implicit None
  Type (NLTE_Atom) :: Atom
  Character (len=256) :: String
  Integer :: fileunit, i, iw, lvlswap, i0, inu
  Logical :: End
  Character (len=256) :: Mytrim
!
  Call Open_file(fileunit, 'ATOM')
  Call Read_next_nocomment(fileunit, String, End)
  Call Tolower(String)
  Atom%element=String(1:2)
  Atom%ion_stage=1
  Read (String(3:255),*,end=100) Atom%ion_stage
100  Call Read_next_nocomment(fileunit, String, End)
!  Read (String,*) Atom%Abund, Atom%AWgt ! These values will be overriden
  Call Read_next_nocomment(fileunit, String, End)
  Read (String,*) Atom%NK, Atom%NLIN, Atom%NCNT, Atom%NFIX
  Call Allocate_Atom_arrays(Atom)
  Do i=1, Atom%NK ! Read level information
     Call Read_next_nocomment(fileunit, String, End)
     Read (String,*) Atom%cm_1(i), Atom%g(i), Atom%lvl_label(i), Atom%ion(i)
  End Do
  Atom%eV(:)=Atom%cm_1(:)*cc*hh/ee
  Do i=1, Atom%NLIN ! Read b-b transition information
     Call Read_next_nocomment(fileunit, String, End)
     Read (String,*) Atom%j(i), Atom%i(i), Atom%f(i), Atom%NQ(i), &
          Atom%QMAX(i), Atom%Q0(i), iw, Atom%GA(i), Atom%GW(i), Atom%GQ(i)
     If (Atom%NQ(i) .gt. MaxNFreqs) then
        Print *,'Line transition ',i,' has too many frquencies'
        Print *,'MaxNFreqs=',MaxNFreqs,' in forward/NLTE/NLTE_atom_struct.f90'
        Stop
     End if
     If (Atom%QMAX(i) .lt. 0 .or. Atom%Q0(i) .lt. 0 .or. iw .lt. 0) then
        Print *,'MULTI feature not supported in model atom'
        Print *,'Line ransition ',i
        Stop
     End if
     If (Atom%j(i) .lt. Atom%i(i)) then ! Swap levels if i .gt. j
        lvlswap=Atom%j(i)
        Atom%j(i)=Atom%i(i)
        Atom%i(i)=lvlswap
     End if
     Atom%krad(Atom%i(i),Atom%j(i))=i
     Atom%krad(Atom%j(i),Atom%i(i))=i
     Atom%Alamb(i)=hh*cc/ee*1.e8/(Atom%eV(Atom%j(i))-Atom%eV(Atom%i(i)))
     Atom%A(i)=Atom%f(i)*6.671e15*Atom%g(Atom%i(i))/Atom%g(Atom%j(i))/&
          Atom%Alamb(i)/Atom%Alamb(i)
     Atom%B(Atom%j(i),Atom%i(i))=Atom%Alamb(i)**3/(2.*hh*cc*1.e24)*Atom%A(i)
     Atom%B(Atom%i(i),Atom%j(i))=Atom%g(Atom%j(i))/Atom%g(Atom%i(i))* &
          Atom%B(Atom%j(i),Atom%i(i))
  End Do
  Do i=1, Atom%NCNT ! Read b-f transition information
     i0=Atom%NLIN+i
     Call Read_next_nocomment(fileunit, String, End)
     Read (String,*) Atom%j(i0), Atom%i(i0), Atom%f(i0), Atom%NQ(i0), &
          Atom%QMAX(i0)
     If (Atom%NQ(i0) .gt. MaxNFreqs) then
        Print *,'Continuum transition ',i,' has too many frquencies'
        Print *,'MaxNFreqs=',MaxNFreqs,' in forward/NLTE/NLTE_atom_struct.f90'
        Stop
     End if
     If (Atom%QMAX(i0) .lt. 0) then
        Do inu=1, Atom%NQ(i0)
           Call Read_next_nocomment(fileunit, String, End)
           Read (String,*) Atom%Q(inu, i0), Atom%AlphaC(inu, i0)
        End do
        Do inu=2, Atom%NQ(i0)
           If (Atom%Q(inu, i0) .gt. Atom%Q(inu-1, i0)) then
              Print *,'Continuum transition: ',i
              Print *,'Wavelengths not decreasing'
              Stop
           End if
        End do
     End if
     If (Atom%j(i0) .lt. Atom%i(i0)) then ! Swap levels if i .gt. j
        lvlswap=Atom%j(i0)
        Atom%j(i0)=Atom%i(i0)
        Atom%i(i0)=lvlswap
     End if
     Atom%krad(Atom%i(i0),Atom%j(i0))=i0
     Atom%krad(Atom%j(i0),Atom%i(i0))=i0
     Atom%Alamb(i0)=hh*cc/ee*1.e8/(Atom%eV(Atom%j(i0))-Atom%eV(Atom%i(i0)))
     Atom%A(i0)=Atom%f(i0)*6.671e15*Atom%g(Atom%i(i0))/Atom%g(Atom%j(i0))/&
          Atom%Alamb(i0)/Atom%Alamb(i0)
  End do
  Do i=1, Atom%NFIX ! Read fixed (radiation temp) b-f transition information
     Call Read_next_nocomment(fileunit, String, End)
     Read (String,*) Atom%jFix(i), Atom%iFix(i), Atom%IPho(i), Atom%A0(i), &
          Atom%TRad(i), Atom%ITRad(i)
     If (Atom%jFix(i) .lt. Atom%iFix(i)) then ! Swap levels if i .gt. j
        lvlswap=Atom%jFix(i)
        Atom%jFix(i)=Atom%iFix(i)
        Atom%iFix(i)=lvlswap
     End if
     If (Atom%ITRad(i) .gt. 3) then
        Print *,'MULTI feature not supported in model atom'
        Print *,'Fixed ransition ',i
        Stop
     End if
  End Do
  If (MaxVal(Atom%NQ(:)) .gt. MaxNFreqs) then 
     Print *,'MaxNFreqs is defined in forward/NLTE/NLTE_atom_struct.f90 as ',MaxNFreqs
     Print *,'Should be at least ',MaxVal(Atom%NQ(:))
     Stop
  End if
! Now store information for calculation of collisional rates
  Call Read_next_nocomment(fileunit, String, End)
  String=Mytrim(String)
  Atom%ColRoutine=String(1:6)
  Atom%NColStrings=0
  Do While (.not. End)
     Call Read_next_nocomment(fileunit, String, End)
     If (.not. End) then 
        Atom%NColStrings=Atom%NColStrings+1
        If (Atom%NColStrings .gt. MaxNColStrings) then 
           print *,'In forward/NLTE/NLTE_atom_struct.f90:'
           print *,'MaxNColStrings=',MaxNColStrings,' (too low)'
           Stop
        End if
        Atom%ColStr(Atom%NColStrings)=String
     End if
  End Do
  Close (fileunit)
  Return
!
End Subroutine Read_atom
