! Solve statistical equilibrium equations and compute NLTE popluations
! Use NLTE%N as starting guess (probably initialized to NLTE%NStar) and
! return the result in the same array
!
Subroutine SolveStat(NLTE, NLTEInput, Atom)
  Use Param_structure
  Use Model_operations
  Use NLTE_Atom_structure
  Use NLTE_inp
  Use NLTE_vars
  Use VirtualFiles
  Implicit None
  Type (NLTE_Atom) :: Atom
  Type (NLTE_variables) :: NLTE
  Type (NLTE_variables), Save :: Saved_NLTE 
  Type (NLTE_input) :: NLTEInput
  Real, Dimension (Atom%NK, NLTE%NDEP) :: E, NOld
  Real, Dimension (Atom%NK, Atom%NK, NLTE%NDEP) :: LU
  Real, Dimension (NLTE%NDEP) :: Phi, Xcont, Scat, Sc, Jnu, Gij, Z, &
       LStar, PhiJ, SBck, Alpha, X, Rnu, S, P, &
       LStMuNu, Jeff
  Real, Dimension (0:NLTE%NDEP) :: iminus, iplus
  Integer, Dimension (NLTE%NDEP) :: largepop
  Logical, Dimension (NLTE%NDEP) :: Depth_converged
  Real :: popmax, TotN, RelChg, Gijk, hnu3c2, hc4p, WQMu, IncidentInt, XC
  Integer :: i, j, iter, itran, idepth, imu, inu, irec, unit
  Logical :: Converged, Newmat, Cont, NGFunc, resetNG
  Logical, Save :: FirstTime=.True.
! Do we need to redo the whole calculation?
  NLTE%Error=.False.
  If (FirstTime) then
     Allocate(Saved_NLTE%Atmo%temp(NLTE%NDEP))
     Allocate(Saved_NLTE%Atmo%v_los(NLTE%NDEP))
     Allocate(Saved_NLTE%Atmo%v_mic(NLTE%NDEP))
     Allocate(Saved_NLTE%N(Atom%NK,NLTE%NDEP))
     Allocate(Saved_NLTE%Source_f(NLTE%NDEP,MaxNFreqs,Atom%NLIN+Atom%NCNT))
  Else
     If (MaxVal(Abs(NLTE%atmo%temp-Saved_NLTE%atmo%temp)) .lt. 1e-5 &
          .and. MaxVal(Abs(NLTE%atmo%v_los-Saved_NLTE%atmo%v_los)) .lt. 1e-5 &
          .and. MaxVal(Abs(NLTE%atmo%v_mic-Saved_NLTE%atmo%v_mic)) .lt. 1e-5) &
     then 

        NLTE%N(:,:)=Saved_NLTE%N(:,:)
        NLTE%Source_f(:,:,:)=Saved_NLTE%Source_f(:,:,:)
        NLTE%Error=Saved_NLTE%Error
        NLTE%atmo=Saved_NLTE%atmo
        If (NLTEInput%Verbose .ge. 2) &
             Print *,'Using previous populations'
        Return
     End if
  End if
!
! First calculation of radiation field, needed to initialize Jnu for Sbck
  Call Radiation
  FirstTime=.False.
  Converged=.False.
  Depth_converged(:)=.False.
  Newmat=.True.
  iter=0
  Do while (iter .lt. NLTEInput%MaxIters .and. .not. Converged)
     iter=iter+1
 ! Initialize arrays
     E(:,:)=0.
     If (Newmat) then
        NLTE%W(:,:,:)=0.
     End if
! Rewind virtual files
     Call RewindVirtualFile('PHI')
     Call RewindVirtualFile('OPC')
     irec=0
! Compute terms for line transitions
     Do itran=1, Atom%NLIN+Atom%NCNT
        i=Atom%i(itran)
        j=Atom%j(itran)
        LStar(:)=0.
        PhiJ(:)=0.
! Source functions, opacities and cross-sections
        Cont=.False.
        If (itran .gt. Atom%NLIN) then
           Cont=.TRUE.
        Else
           Gijk=Atom%g(i)/Atom%g(j)
           hnu3c2=Atom%A(itran)/Atom%B(j,i)
           Z(:)=NLTE%N(i,:)-Gijk*NLTE%N(j,:)
           Gij(:)=Gijk
           NLTE%Sl(:,itran)=hnu3c2*NLTE%N(j,:)*Gijk/Z(:)
        End if
        Do inu=1, Atom%NQ(itran)
           If (Cont .or. inu .eq. 1) Call ReadX(Xcont, Scat, Sc)
           hc4p=hh*cc*1.e-5/NLTEInput%QNORM/4./Pi
           irec=irec+1
           Call ReadJ(irec, Jnu)
           SBck(:)=Sc(:)+Scat(:)*Jnu(:)
           Jnu(:)=0. ! Reset Jnu to recompute at this frequency
           If (Cont) then
!              Do idepth=1, NLTE%NDEP ! Do not add twice the same continuum
!                 XC=XCont(idepth)
!                 XC=MAX(1.e-3*XCont(idepth),XCont(idepth)- &
!                      Z(idepth)*Alpha(idepth)/NLTE%Xnorm(idepth))
!                 XCont(idepth)=XC
!              End do
              hnu3c2=2.*hh*Atom%FRQ(inu,itran)/cc*Atom%FRQ(inu,itran)/cc * &
                   Atom%FRQ(inu,itran)
              Gij(:)=NLTE%NStar(i,:)/NLTE%NStar(j,:)*Exp(-hh* &
                   Atom%FRQ(inu,itran)/bk/NLTE%Atmo%Temp(:))
              Alpha(:)=Atom%AlphaC(inu,itran)
              Z(:)=NLTE%N(i,:)-Gij(:)*NLTE%N(j,:)
              NLTE%Sl(:,itran)=hnu3c2*NLTE%N(j,:)*Gij(:)/Z(:)
           End if
           Do imu=1, NLTEInput%NMU
              WQMu=Atom%WQ(inu,itran)*NLTE%WMu(imu)
              If (.not. Cont .and. (.not. NLTEInput%VelFree .or. imu .eq. 1)) then
                 Call ReadP(Phi)
                 Alpha(:)=Atom%B(i,j)*Phi(:)*hc4p
              End if
              If (.not. NLTEInput%VelFree .or. imu .eq. 1) then
                 X(:)=Z(:)*Alpha(:)/NLTE%Xnorm(:)+Xcont(:)
                 Rnu(:)=Xcont(:)/X(:)
                 S(:)=(1.-Rnu(:))*NLTE%Sl(:,itran)+Rnu(:)*SBck(:)
              End if
              IncidentInt=0. ! No incident radiation
! Compute monochromatic radiation field and local operator
!              If (imu .eq. 1) NLTE%Source_f(:,inu,itran)=S(:)
              If (imu .eq. 1) NLTE%Source_f(:,inu,itran)=NLTE%Sl(:,itran)
              Call FormalSolution(NLTE, imu, X, S, RNu, P, LStMuNu)
              Jnu(:)=Jnu(:)+NLTE%WMu(imu)*P(:)
              PhiJ(:)=PhiJ(:)+WQMu*Phi(:)*P(:)
              If (Cont) then
                 LStar(:)=LStar(:)+NLTE%WMu(imu)*LStMuNu(:)
              Else
                 LStar(:)=LStar(:)+WQMu*LStMuNu(:)*Phi(:)
              End if
           End do ! Do in imu

           If (Cont) then
              Jeff(:)=Jnu(:)-LStar(:)*NLTE%Sl(:,itran)
              If (MinVal(Jeff) .lt. 0) then 
                 Do idepth=1, NLTE%NDEP
                    If (Jeff(idepth) .lt. 0) Jeff(idepth)=Jnu(idepth)
                 End do
              End if
              NLTE%W(i,j,:)=NLTE%W(i,j,:)+Atom%WQ(inu,itran)/hc4p/ &
                   Atom%FRQ(inu,itran)*Atom%FRQ(0,itran)*Gij(:)*Alpha(:)*&
                   (hnu3c2*(1.-LStar(:))+Jeff(:))
              NLTE%W(j,i,:)=NLTE%W(j,i,:)+Atom%WQ(inu,itran)/hc4p/ &
                   Atom%Frq(inu,itran)*Atom%FRQ(0,itran)*Alpha(:)*Jeff(:)
           End if
           Call WriteJ(irec,Jnu)
        End do ! Do in inu
        If (.not. Cont) then
           If (MaxVal(LStar(:)) .gt. 1) then
              Do idepth=1, NLTE%NDEP
                 If (LStar(idepth) .gt. 1) LStar(idepth)=1.
              End do
           End if
           If (MinVal(LStar(:)) .lt. 1) then
              Do idepth=1, NLTE%NDEP
                 If (LStar(idepth) .lt. 0) LStar(idepth)=0.
              End do
           End if
           PhiJ(:)=PhiJ(:)*NLTE%WPhi(:,itran)
           Jeff(:)=Phij(:)-LStar(:)*NLTE%Sl(:,itran)
           if (MinVal(Jeff(:)) .lt. 0) then
              Do idepth=1, NLTE%NDEP
                 If (Jeff(idepth) .lt. 0) then
                    If (Jeff(idepth) .le. 0.1*PhiJ(idepth)) then
                       Jeff(idepth)=0.
                    Else
                       Jeff(idepth)=PhiJ(idepth)
                    End if
                 End if
              End do
           End if
           NLTE%W(i,j,:)=NLTE%W(i,j,:)+(Atom%A(itran)*(1.-LStar(:))+ &
                Atom%B(j,i)*Jeff(:))
           NLTE%W(j,i,:)=NLTE%W(j,i,:)+Atom%B(i,j)*Jeff(:)
        End if ! If .not. Cont
     End do ! Do in itran
! Add collisional and fixed rates (C and F have inverted indeces wrt W)
     Do i=1, Atom%NK
        Do j=1, Atom%NK
           NLTE%W(i,j,:)=NLTE%W(i,j,:)+NLTE%C(j,i,:)+NLTE%F(j,i,:)
        End do
     End do
! The sum over the i-th column gives the total rate out of the i-th level
     Do i=1, Atom%NK
        Do j=1, i-1
           NLTE%W(i,i,:)=NLTE%W(i,i,:)-NLTE%W(j,i,:)
        End do
        Do j=i+1, Atom%NK
           NLTE%W(i,i,:)=NLTE%W(i,i,:)-NLTE%W(j,i,:)
        End do
     End do
!
     largepop(:)=NLTEInput%ISum
     If (NLTEInput%ISum .eq. 0) then
! Find out which level has the largest population at each depth-point
        Do idepth=1, NLTE%NDEP
           popmax=NLTE%N(1,idepth)
           largepop(idepth)=1
           Do i=2, Atom%NK
              If (NLTE%N(i,idepth) .gt. popmax) then
                 popmax=NLTE%N(i,idepth)
                 largepop(idepth)=i
              End if
           End do
        End do
     End if
! Particle conservation equation
     E(:,:)=0.
     Do idepth=1,NLTE%NDEP
        NLTE%W(largepop(idepth),:,idepth)=1.
        E(largepop(idepth),idepth)=Sum(NLTE%N(:,idepth))
     End do
!
! Ok. Matrices are now ready. Update populations
!
     NOld(:,:)=NLTE%N(:,:)
     Do idepth=1,NLTE%NDEP
        If (.not. Depth_converged(idepth)) then
           E(:,idepth)=E(:,idepth)-MatMul(NLTE%W(:,:,idepth), &
                NLTE%N(:,idepth))
           Do j=1, Atom%NK
              NLTE%W(:,j,idepth)=NLTE%W(:,j,idepth)*NLTE%N(j,idepth)
           End do
           If (NewMat) then
              Call EneQ(Atom%NK,1,NLTE%W(:,:,idepth),E(:,idepth),.TRUE.)
           Else
              Call EneQ(Atom%NK,1,LU(:,:,idepth),E(:,idepth),.FALSE.)
           End if
           NLTE%N(:,idepth)=NOLD(:,idepth)*(1.+E(:,idepth))
        End if
     End do
!
     If (MinVal(NLTE%N(:,:)).LT.0) then
        Print *,'Warning! Negative populations!'
        NLTE%Error=.True.
        NLTE%N(:,:)=ABS(NLTE%N(:,:))
     End if
     RelChg=MaxVal(Abs((NOLD(:,:)-NLTE%N(:,:))/NLTE%N(:,:)))

     Do idepth=1, NLTE%NDEP
        If (MaxVal(Abs((NOLD(:,idepth)-NLTE%N(:,idepth))/ &
             NLTE%N(:,idepth))) .lt. NLTEInput%elim2) then 
           Depth_converged(idepth)=.True. 
           LU(:,:,idepth)=NLTE%W(:,:,idepth)
        End if
     End do
! NG acceleration
     If (iter .eq. 1) & ! Reset NG acceleration
          resetNG=NGFunc(NLTE%N, Atom%NK, NLTE%NDEP, 3, .true.)
     If (RelChg .lt. 1 .and. NLTEInput%NGacc) then
        If (NGFunc(NLTE%N, Atom%NK, NLTE%NDEP, 3, .false.)) then
           If (NLTEInput%Verbose .ge. 2) &
                Print *,'NG acceleration'
        End if
     End if
     If (minval(nlte%n) .lt. 0) then
        print *,'negativ pop after NG!!'
        NLTE%Error=.True.
        NLTE%N=Abs(NLTE%N)
     end if
!
     If (NLTEInput%Verbose .ge. 3) &
          Print *,'POPULATIONS RELATIVE CHANGE: ', RelChg
     If (RelChg .lt. NLTEInput%elim1 .and. NewMat) THEN
        NewMat=.FALSE.
        LU(:,:,:)=NLTE%W(:,:,:)
     End if
! Stop?
     If (RelChg .lt. NLTEInput%elim2) Converged=.TRUE.
!
  End do ! Do while (iterations)
  If (NLTEInput%Verbose .ge. 2) &
       Print *,'Populations maximum relative change: ', RelChg
!
  If (RelChg .lt. 1e-2) then
     Saved_NLTE%N(1:Atom%NK,1:NLTE%NDEP)=NLTE%N(1:Atom%NK,1:NLTE%NDEP)
     Saved_NLTE%Source_f(:,:,:)=NLTE%Source_f(:,:,:)
     Saved_NLTE%Error=NLTE%Error
     Saved_NLTE%atmo=NLTE%atmo
  Else
     NLTE%Error=.True.
  End if
!
  If (NLTEInput%Write) then
     Call Open_file_unform(unit, 'ltepop.dat')
     Write (unit) NLTE%NStar(:,:)
     Close (unit)
     Call Open_file_unform(unit, 'pop.dat')
     Write (unit) NLTE%N(:,:)
     Close (unit)
  End if
!
  Return
!
  contains
    Subroutine Radiation

      integer :: itran, inu
! Computes Jnu for all transitions, freqs and angles and store it.
! This routine is analogous to the computation in solvestat, except
! that it doesn't compute LStar, Jeff, etc
! Rewind virtual files
      Call RewindVirtualFile('PHI')
      Call RewindVirtualFile('OPC')
      irec=0
! Compute terms for line transitions
      Do itran=1, Atom%NLIN+Atom%NCNT
         i=Atom%i(itran)
         j=Atom%j(itran)
! Source functions, opacities and cross-sections
         Cont=.False.
         If (itran .gt. Atom%NLIN) then
            Cont=.TRUE.
         Else
            Gijk=Atom%g(i)/Atom%g(j)
            hnu3c2=Atom%A(itran)/Atom%B(j,i)
            Z(:)=NLTE%N(i,:)-Gijk*NLTE%N(j,:)
            Gij(:)=Gijk
            NLTE%Sl(:,itran)=hnu3c2*NLTE%N(j,:)*Gijk/Z(:)
         End if
         Do inu=1, Atom%NQ(itran)
            If (Cont .or. inu .eq. 1) Call ReadX(Xcont, Scat, Sc)
            hc4p=hh*cc*1.e-5/NLTEInput%QNORM/4./Pi
            irec=irec+1
            SBck(:)=Sc(:)
            Jnu(:)=0.
            If (Cont) then
!               Do idepth=1, NLTE%NDEP ! Do not add twice the same continuum
!                  XC=XCont(idepth)
!                  XC=MAX(1.e-3*XCont(idepth),XCont(idepth)- &
!                       Z(idepth)*Alpha(idepth)/NLTE%Xnorm(idepth))
!                  XCont(idepth)=XC
!               End do
               hnu3c2=2.*hh*Atom%FRQ(inu,itran)/cc*Atom%FRQ(inu,itran)/cc * &
                    Atom%FRQ(inu,itran)
               Gij(:)=NLTE%NStar(i,:)/NLTE%NStar(j,:)*Exp(-hh* &
                    Atom%FRQ(inu,itran)/bk/NLTE%Atmo%Temp(:))
               Alpha(:)=Atom%AlphaC(inu,itran)
               Z(:)=NLTE%N(i,:)-Gij(:)*NLTE%N(j,:)
               NLTE%Sl(:,itran)=hnu3c2*NLTE%N(j,:)*Gij(:)/Z(:)
            End if
            NLTE%Source_f(:,inu,itran)=S(:)
            Do imu=1, NLTEInput%NMU
               If (.not. Cont .and. (.not. NLTEInput%VelFree .or. imu .eq. 1)) then
                  Call ReadP(Phi)
                  Alpha(:)=Atom%B(i,j)*Phi(:)*hc4p
               End if
               If (.not. NLTEInput%VelFree .or. imu .eq. 1) then
                  X(:)=Z(:)*Alpha(:)/NLTE%Xnorm(:)+Xcont(:)
                  Rnu(:)=Xcont(:)/X(:)  
                  S(:)=(1.-Rnu(:))*NLTE%Sl(:,itran)+Rnu(:)*SBck(:)
               End if
               Iminus(0)=0. ! No incident radiation
! Compute monochromatic radiation field and local operator
               Call FormalSolution(NLTE, imu, X, S, RNu, P, LStMuNu)
!
               Jnu(:)=Jnu(:)+NLTE%WMu(imu)*P(:)
            End do ! Do in imu
            Call WriteJ(irec,Jnu)
         End do ! Do in inu
     End do ! Do in itran
     Return
!
   End Subroutine Radiation
!
 End Subroutine SolveStat
