! This routine convolves the profile with a Gaussian whose width is the
! macroturbulent velocity.
!
Subroutine Convolve_profile(Params, Region, Atmo, Profile)
  Use Param_structure
  Use Model_structure
  Use Line_data_structure
  Implicit None
  Type (Parameters) :: Params
  Type (Region_data), dimension (Params%n_regions) :: Region
  Type (model) :: Atmo
  Real, Dimension (Params%n_data) :: Profile, data, respns, gauss, ans
  Integer :: i, n, m, mmax, n2, iregion, istokes, iwave, istart, iunit
  Real :: x, Wave
  Logical, Save :: FirstTime=.True., ExistsInstrumProf=.False.
  Real, Dimension(:,:), Pointer, Save :: Instrum_prof
  Integer, Dimension(:), Pointer, Save :: mm
  Logical :: Exists
!
  If (FirstTime) then 
     FirstTime=.False.
     ! Read instrumental profile from file?
     Inquire (File='Instrumental_profile.dat', Exist=Exists)
     If (Exists) then
        ExistsInstrumProf=.True.
        If (Params%Printout .ge. 1) &
           Print *,'Reading Instrumental_profile.dat'
        Allocate(mm(Params%n_regions))
        ! File format is: 
        !  For each region:
        !  First line -> number of wavelength points. It must be odd and
        !     cannot be greater than the number of wavelengths in the grid 
        !     file
        !  Each line -> Instrument profile value, centered at first point
        !     then from m/2+2 to m is the left part of the profile
        Call Open_file(iunit,'Instrumental_profile.dat')
        mmax=0
        Do n=1, Params%n_regions ! Find region with most wavelengths
           Read (iunit,*) m
           mm(n)=m
           If (m .gt. mmax) mmax=m
           Do i=1, m
              Read (iunit,*) x
           End do
        End do
        Allocate(Instrum_prof(Params%n_regions,mmax))
        Close(iunit)
        Call Open_file(iunit,'Instrumental_profile.dat')
        Do n=1, Params%n_regions
           Read (iunit,*) m
           Do i=1, m
              Read (iunit,*) Instrum_prof(n, i)
           End do
           If (Mod(m,2) .ne. 1) then ! If even number of points, make it odd
              mm(n)=mm(n)-1
              Instrum_prof(n,m)=0.
           End if
        End do
        Close (iunit)
     End if
  End if
!
  If (Atmo%v_mac .eq. 0 .and. .not. ExistsInstrumProf) Return
!
  istart=1 ! Index to the first point of the line
  Do iregion=1, Params%n_regions
     If (ExistsInstrumProf) then
        ! Use instrumental profile from file
        m=mm(iregion)
        gauss(1:m)=Instrum_prof(iregion,1:m)
     Else
        ! Construct the gaussian
        Wave=Region(iregion)%First_wlength + &
             Region(iregion)%nwavelengths/2.*Region(iregion)%Wave_step
        m=3.*(Atmo%v_mac/cc*Wave/Region(iregion)%Wave_step*Region(iregion)%Macro_enh)
        m=(m/2.)*2+1 ! Make sure m is an odd integer
        gauss(1)=1. ! To prevent the case where v_mac .eq. 0.
        If (m .gt. 1) then
           Do i=1, m/2+1 ! Positive x
              x=(i-1)*Region(iregion)%Wave_step/ &
                   (Atmo%v_mac*Region(iregion)%Macro_enh/cc*Wave)
              gauss(i)=Exp(-x*x)
           End do
        End if
        Do i=m/2+2, m ! Negative x
           gauss(i)=gauss(m-i+1)
        End do
     End if
     gauss(1:m)=gauss(1:m)/Sum(gauss(1:m)) ! Normalization
!
! Construct the profile to convolve (one for each Stokes profiles)
!
     n=Region(iregion)%nwavelengths ! Number of points
     If (m .ge. n) then
        Print *,'Error in Convolve_profile. m .ge. n !!!'
     Else
! Find nearest power of 2
        i=1
        Do while (2.**i .lt. n)
           i=i+1
        End do
        n2=2.**i
        If (n2 .gt. Params%n_data/2) then
           Print *,'Cannot convolve. Aborting ...'
           Stop
        End if
! 
        Do istokes=1, 4
           Do iwave=1, n
              data(iwave)=Profile(istart+(iwave-1)*4+istokes-1)
           End do
           data(n+1:n2)=data(n)
!
! Perform the convolution
!
           respns(1:m)=gauss(1:m)
           Call convlv(data(1:n2), n2, respns(1:n2), m, 1, ans(1:2*n2))
!
! Store the result back in Profile
!
           Do iwave=1, n
              Profile(istart+(iwave-1)*4+istokes-1)=ans(iwave)
           End do
        End do ! Next Stokes parameter
     End if
     istart=istart + 4*n ! Update istart to point to the next region
  End do ! Next spectral region
!
! Done!
!
  Return
End Subroutine Convolve_profile
