! This subroutine calculates the derivatives of a function y assuming a
! parabolic behavior between i-1, i and i+1 (being i the point where the
! derivative is to be evaluated). The derivatives at the boundaries are
! computed assuming a linear behavior. The vector x does not need to be
! equispaced.
!
Subroutine Parab_deriv(npoints, x, y, dy)
  Implicit None
  Real, Dimension (npoints) :: x, y, dy
  Real :: a, b, c, den
  Integer :: npoints, i
!
! Linear interpolation for the boundaries
!
  dy(1)=(y(2)-y(1))/(x(2)-x(1))
  dy(npoints)=(y(npoints)-y(npoints-1))/(x(npoints)-x(npoints-1))
!
! Parabolic interpolation for all the other points
!
  Do i=2,npoints-1
     den=(x(i)-x(i-1))*(x(i)-x(i+1))*(x(i-1)-x(i+1))
     a = (x(i+1)*(y(i-1)-y(i)) + x(i-1)*(y(i)-y(i+1)) + &
          x(i)*(y(i+1)-y(i-1)))/den
     b = (x(i+1)*x(i+1)*(y(i)-y(i-1)) + x(i)*x(i)*(y(i-1)-y(i+1)) + &
          x(i-1)*x(i-1)*(y(i+1)-y(i)))/den
!
!    y = a x^2 + bx + c => y' = 2 a x + b
!
     dy(i)=2.*a*x(i)+b
  End do
  Return
End Subroutine Parab_deriv
