      SUBROUTINE READION(N_elements, Atom_char, At_abund, ifile)
C
C***********************************************************IAC, 08.06.97
C                                                   Nataliya G. Shchukina                                                     
C
C   Read atomic data from file ION.DAT  
C   
C   Data are needed for solution of ionization equilibrium
C
C**********************************************************************
C
C  There are TWO choices for 
C  solution of ionization equilibrium equations:
C----------
C Choice 1:
C----------
C
C  IHTOT > 0 - density RHO and total number of hydrogen atoms HTOT
C              are known
C              from gas pressure PG and electron pressure PE 
C
C              subroutine STATP is called.
C
C  IHTOT = 1   without molecules 
C          2   with    molecules H2+ and H2.
C
C----------
C Choice 2:
C----------
C
C   IHTOT < 0 - density RHO and total number of hydrogen atoms HTOT 
C               are determined with new
C               chemical composition.
C
C              subroutine STATM is called.
C
C  IHTOT =-1   without molecules
C  IHTOT =-2   with    molecules H2+ and H2.
C
C

C   Contributitors in in the electron density NE: 
C   H, Si, C, Mg, Al, Fe, He, N, O, Ne, Na, P, S, Ar, Ca, Cr, Mn.
C

C********************************************************************
C
C  PARAMAX contains dimensions of arrays 
C
C MODIFIED BY HECTOR (DEC 99) TO TAKE EXTERNAL ABUNDANCES. THESE ARE
C NO LONGER IN THE FILE ion.dat. OTHER PARAMETERS, HOWEVER
C (PARTITION FUNCTIONS, ATOMIC WEIGHT, ETC), ARE STILL TAKEN FROM THE
C FILE, BUT THESE ARE NORMALLY VALUES THAT CAN BE HARDWIRED WITHOUT 
C MUCH TROUBLE.
      
C                INCLUDE 'PARAMAX'
      PARAMETER (MDEP=1,MK=1,MFT=1)  
                INCLUDE 'CCONST'
C
                                                          
      DIMENSION NELEM(1),KU(1),KEL(1)                                  
      DIMENSION ELMENT(25),ALFA(25),AS(25),CHIO(25)                     
      DIMENSION NBELO1(2,25),NTO1(2,25)                
      REAL*8 NLEVEL   
      DIMENSION TO1(248),UO1(248),NLEVEL(30),EXO1(30),                  
     ,     GO1(30),BELEM(MDEP,30)             
      Character*2 Atom_char(N_elements), tmpchar
      Real At_abund(N_elements), tmpreal
      Logical found
      Integer ifile
      
      COMMON V(10300)
C
      COMMON/BION/ BM(MDEP,30)
C 
      COMMON/ATWT/ ATW,SUM
C                                 
      Equivalence (tmpchar, tmpreal)
      EQUIVALENCE(V(2000),NELEM(1)),(V(2001),ELMENT(1))                 
      EQUIVALENCE(V(2026),ALFA(1)),(V(2051),AS(1)),(V(2076),CHIO(1))    
      EQUIVALENCE(V(2101),NBELO1(1)),(V(2151),NTO1(1))                  
      EQUIVALENCE(V(2201),KU(1)),(V(2202),TO1(1)),(V(2450),UO1(1))      
      EQUIVALENCE(V(2698),KEL(1)),(V(2699),
     ,NLEVEL(1)),(V(2759),EXO1(1)),
     ,(V(2789),GO1(1))
C 
c         PRINT *,' READION read atomic data from file ION.DAT' 
      
                                                          
         DO I=1,30                                                    
           DO J=1,MDEP              
             BM(J,I)=1.0
           End do
         End do
         DO I=1,30
           DO J=1,MDEP
             BELEM(J,I)=1.
           End do
         End do
              
      READ (ifile,71) NELEM(1),KEL(1),KU(1)
C      PRINT *,NELEM(1),KEL(1),KU(1)

   71 FORMAT(3I5)
                                          
      IF(NELEM(1).LE.25) GO TO 70                                       
      PRINT *,'INPUT ERROR IN ATMOS: NELEM HAS TO BE < OR = 25'        
      STOP                                                              
   70 CONTINUE                                                          
      IF(KEL(1).LE.6) GO TO 74                                          
      PRINT *,'INPUT ERROR IN ATMOS: KEL HAS TO BE < OR = 6'          
      STOP                                                   
   74 CONTINUE                                                          
      IF(KU(1).LE.6) GO TO 170                                          
      PRINT *,'INPUT ERROR IN ATMOS: KU HAS TO BE < OR = 6'           
      STOP                                                             
  170 CONTINUE
                                                            
      KBO=0                                                            
      NTO=0                                                            
      NLMM=NELEM(1)                                                      
      XMUM=0.
      SUM=0.
      DO IE=1,NLMM 
C----------------------------------------------------------------
C ELMENT is the name of chemical element in the Mendeleev's table
C ALFA   is an abundance  relatively hydrogen 
C AS     is an atomic weight
C CHIO   is ionization potential
C
C TO1 and UO1 are the temperature and partition function for neutral atom
C NTO1 is tne number of the temperature points
C
C TO2, UO2, NTO2 are the same for ion
C 
C NBELO1 = 0 for LTE case
C
C if NBELO1 NE 0 you have to read:
C
C NLEVEL - name of the level;
C EXO1   - excitation potential of the level;
C GO1    - statistical weight;
C BELEM  - departure coefficients for the grid of depth points
C
C NB!!!
C Natasha comments the NLTE case
C----------------------------------------------------------------                                                      
      READ (ifile,75) ELMENT(IE),ALFA(IE),AS(IE),CHIO(IE)
C      PRINT *,ELMENT(IE),ALFA(IE),AS(IE),CHIO(IE)
      Found=.FALSE.
      Do ind=1, N_elements
        tmpreal=ELMENT(IE)
        If (Atom_char(ind) .eq. tmpchar) then
          Found=.TRUE.
          ALFA(IE)=10.**(At_abund(ind)-12.)
        End if
      End do
      If (.not.Found) then
        Print *,'Element ',ELMENT(IE),' in ion.dat not found in the'
        Print *,'atomic data table!! Aborting ...'
        Stop
      End if
                      
   75 FORMAT(A4,3E10.3)                                                 
       XMUM=XMUM+ALFA(IE)*AS(IE)
       SUM=SUM+ALFA(IE)
      DO IO=1,2                                                     
      READ (ifile,77) NBELO1(IO,IE),NTO1(IO,IE) 
C      PRINT *,NBELO1(IO,IE),NTO1(IO,IE) 
                              
   77 FORMAT(2I5)                                                       
      NB=NBELO1(IO,IE)+KBO                                              
      NTE=NTO1(IO,IE)+NTO                                               
      NTO=NTO+1                                                       
      DO IU=NTO,NTE                                                
      READ (ifile,79) TO1(IU),UO1(IU)
C      PRINT *,TO1(IU),UO1(IU)
                                     
   79 FORMAT(E6.0,E10.3)                                               
      End do
      NTO=NTE                                                          
       IF(NBELO1(IO,IE).EQ.0) GO TO 104                                 
       KBO=KBO+1                                                       
       DO IB=KBO,NB                                                
       READ (ifile,81) NLEVEL(IB),EXO1(IB),GO1(IB)                             
   81  FORMAT(A8,E10.3,F5.0)                                            
       DO ID=1,ND,5                                                 
       READ (ifile,83) BELEM(ID,IB)
C       ,                                           
C      ,BELEM(ID+1,IB),BELEM(ID+2,IB),BELEM(ID+3,IB),BELEM(ID+4,IB)      
C
   83  FORMAT(5E15.7)                                                   
       End do
       End do
       KBO=NB                                                            
 104   CONTINUE                                                        
       End do
       End do

C  ATW - mean atomic weight MU

         ATW=XMUM/SUM
C         PRINT *,ATW,SUM
        RETURN
        END        
