! This subroutine takes a structure Atom and computes the collisional rates
!
Subroutine Collisions(NLTE, Atom)
  Use NLTE_Atom_structure
  Use NLTE_vars
  Use Param_structure
  Implicit None
  Type (NLTE_Atom) :: Atom
  Type (NLTE_variables) :: NLTE
!
  If (Atom%ColRoutine .eq. 'CA2COL') then
     Call Ca2Col(Atom, NLTE)
  Else if (Atom%ColRoutine .eq. 'GENCOL') then
     Call GenCol(Atom, NLTE)
  Else if (Atom%ColRoutine .eq. 'HCOL') then
     Call HCol(Atom, NLTE)
  Else
     Print *,'Collisional routine not supported:',Atom%ColRoutine
     Stop
  End if

End Subroutine Collisions

Subroutine Ca2Col(Atom, NLTE)
! Adapted from MULTI
 Use Param_structure
 Use NLTE_Atom_structure
 Use NLTE_vars
 Implicit None
 Type (NLTE_Atom) :: Atom
 Type (NLTE_variables) :: NLTE
 Real, Dimension(:), Allocatable :: CE, CI
 Real, Dimension(:,:,:), Allocatable :: C
 Integer :: i, j, idepth, imax
 Real :: DC
!
 Allocate (C(Atom%NK, Atom%NK, NLTE%NDEP))
 Allocate (CE(Atom%NK-1))
 Allocate (CI(Atom%NK-1))
 C(:,:,:)=0.
!
! Start with de-excitation rates first
!
!
! Bound-bound collisions with electrons
!
 Do j=2, Atom%NK-1
    imax=j-1
    Read (Atom%ColStr(j-1),*) CE(1:imax)
    Do i=1, j-1
       Do idepth=1, NLTE%NDEP
          C(j,i,idepth)=CE(i)*Sqrt(5000./NLTE%Atmo%Temp(idepth))*NLTE%Atmo%ne(idepth)
          C(i,j,idepth)=C(j,i,idepth)*NLTE%Nstar(j,idepth)/NLTE%Nstar(i,idepth)
       End do
    End do
 End do
!
! Collisions with neutral hydrogen
! (Only for Ca 6-level atom, we consider leves 2-3 and 4-5)
!
 If (Atom%NK .eq. 6) then
    Do idepth=1, NLTE%NDEP
       DC=1.5E-9*(NLTE%Atmo%Temp(idepth)/1000.)**0.33*NLTE%Atmo%nH(idepth)
       C(4,5,idepth)=C(4,5,idepth)+DC
       C(2,3,idepth)=C(2,3,idepth)+DC
       C(5,4,idepth)=C(5,4,idepth)+DC*NLTE%Nstar(4,idepth)/NLTE%Nstar(5,idepth)
       C(3,2,idepth)=C(3,2,idepth)+DC*NLTE%Nstar(2,idepth)/NLTE%Nstar(3,idepth)
    End do
 Else
    Print *,'Collisions with H ignored for this atom'
 End if
!
! Bound-free rates
!
 If (Atom%ion(6) .eq. Atom%ion(1)+1) then
    Read (Atom%ColStr(Atom%NK-1),*) CI(1:Atom%NK-1)
    Do i=1, Atom%NK-1
       Do idepth=1,NLTE%NDEP
          C(i,Atom%NK,idepth)=CI(i)*NLTE%Atmo%ne(idepth)*SQRT(NLTE%Atmo%Temp(idepth))*EXP(-ee/bk/NLTE%Atmo%Temp(idepth)* &
               (Atom%eV(Atom%NK)-Atom%eV(i)))
          C(Atom%NK,i,idepth)=C(i,Atom%NK,idepth)*NLTE%Nstar(i,idepth)/NLTE%Nstar(Atom%NK,idepth)
       End do
    End do
 Else
    Print *,'forward/NLTE/collisions: This atom does not have the expected structure...'
    Stop
 Endif
!
 NLTE%C=C
 Deallocate (C,CE,CI)
!
 Return
End Subroutine Ca2Col

Subroutine GenCol(Atom, NLTE)
! Adapted from MULTI
! Note: Some features have been removed in order to port that routine here!!
!
 Use Param_structure
 Use NLTE_Atom_structure
 Use NLTE_vars
 Type (NLTE_Atom) :: Atom
 Type (NLTE_variables) :: NLTE
!
!  GENERAL ROUTINE FOR COMPUTING COLLISIONAL RATES (VERSION MAR '89)    
!  FORMAT IS EITHER GIVEN IN TERMS OF A COLLISION STRENGTH OMEGA        
!         OR                          A CE(TE) RATE                     
!                                                                       
!  OMEGAS ARE USED FOR CHARGED IONS IN GENERAL,                         
!  SINCE THEY REMAIN ROUGHLY CONSTANT WITH TEMPERATURE.                 
!  RATE IS PROP. TO 1/SQRT(TE) * EXP(DELTA-E)                           
!                                                                       
!  CE(TE) VALUES ARE USED LARGELY FOR NEUTRALS FOR SAME REASON.         
!  RATE IS PROP. TO SQRT(TE) * EXP(DELTA-E)                             
!                                                                       
!  CI(TE) VALUES ARE USED FOR IONIZATION RATES: THESE HAVE A            
!         SQRT(TE) * EXP(DELTA-E) DEPENDENCE AS FOR CE(TE)              
!                                                                       
!  ORIGINALLY CODED BY P.G. JUDGE,  APRIL 2ND, 1987                     
!  MODIFIED FOR FORTRAN-77 STANDARD BY P.G. JUDGE, MAY 21ST 1987        
!:                                                                      
!: GENCOL 87-05-21  NEW ROUTINE: (PHILIP JUDGE)                         
!:        GENERAL COLLISONAL ROUTINE READING TABLES                     
!:                                                                      
!:        89-03-24  MODFICATIONS: (PHILIP JUDGE)                        
!:        NEW INPUT 'CP' AND 'CH' ADDED FOR INELASTIC COLLISIONS        
!:        WITH NEUTRAL AND IONIZED HYDROGEN.                            
!:        'SEMI ' CHANGED SO THAT THE ABSORPTION OSCILLATOR STRENGTH    
!:        IS REQUIRED FOR INPUT.                                        
!:                                                                      
!:        89-06-07  MODIFICATIONS: (MATS CARLSSON)                      
!:        DO LOOP VARIABLE IT CHANGED TO JT IN ORDER NOT TO CONFLICT    
!:        WITH THE MAIN ITERATION VARIABLE IT IN ITER                   
!:                                                                      
!:        90-04-10  MODIFICATIONS: (PHILIP JUDGE)                       
!:        LOW TEMPERATURE DIELECTRONIC RECOMBINATION OPTION ADDED       
!:        FOLLOWING NUSSBAUMER AND STOREY (A+A  )                       
!:        NOTE THAT ONLY THE RATE DOWNWARD IS ADDED TO THE FIXED RATE   
!:                                                                      
!:        90-08-27  MODIFICATIONS: (MATS CARLSSON)                      
!:        DO LOOP 1 REPLACED BY A GOTO TO REMOVE RESTRICTION ON         
!:        NUMBER OF LINES WITH COLLISIONAL DATA                         
!:                                                                      
!:        92-08-10  MODIFICATIONS (MATS CARLSSON)                       
!:        INTEGER ARRAYS WITH ELEMENT NAME CHANGED TO CHARACTER ARRAYS  
!:                                                                      
!:        92-09-07  MODIFICATIONS (MATS CARLSSON)                       
!:        ADDED OPTIONS RECOMB AND ALPHA                                
!:                                                                      
!:        94-01-24  MODIFICATIONS (PHILIP JUDGE)                        
!:        AR85 OPTIONS ADDED.  ARNAUD AND ROTHENFLUG 1985               
!:                                                                      
!:        94-02-22  MODIFICATIONS (PHILIP JUDGE)                        
!:        AR85-CEA FORMALISM ADDED                                      
!:                                                                      
!:        94-03-08  MODIFICATIONS (PHILIP JUDGE)                        
!:        AR85-CH, AR85-CH+, AR85-CHE, AR85-CHE+, FORMALISM ADDED       
!:                                                                      
!:        95-08-23  MODIFICATIONS (MATS CARLSSON)                       
!:        THERMAL P-FUNCTION FOR SEMI-EMPIRICAL RATES RENAMED TO PSEMI  
!:                                                                      
      PARAMETER (MTGRD = 50, MSHELL = 5)                                 
!                                                                       
!  LOCAL VARIABLES: UP TO MTGRD POINTS IN TEMPERATURE GRID ALLOWED      
!  THIS CAN BE INCREASED                                                
!                                                                       
      DIMENSION TGRID (MTGRD) , CGRID (MTGRD)                            
      DIMENSION CDI (5, MSHELL)                                          
      Real, Parameter :: EK=ee/bk
      Real, Dimension(:,:,:), Allocatable :: C
      Real, Dimension(:), Allocatable :: TEMP, NE
      Real, Dimension(:,:), Allocatable :: NH
      CHARACTER(20) KEY                                                  
!  ALPRD IS TRUE IF INITIA HAS BEEN CALLED TO CALCULATE ALPHA COLLISIONS
      LOGICAL ALPRD                                                      
      DATA ALPRD / .FALSE. /                                             
      SAVE ALPRD    
!
 Allocate (C(Atom%NK, Atom%NK, NLTE%NDEP))
 C(:,:,:)=0.
 NK=Atom%NK
 NDEP=NLTE%NDEP
 Allocate (Temp(NDEP))
 Allocate (NE(NDEP))
 Allocate (NH(6,NDEP))
 Temp(:)=NLTE%Atmo%Temp(:)
 NE(:)=NLTE%Atmo%El_p(:)/bk/Temp(:)
 NH(:,:)=0. 
 NH(1,:)=NLTE%Atmo%nH(:) ! Neutral H
 NH(6,:)=NLTE%Atmo%nHplus(:) ! Ionized H
!                                                                       
!  ASSUME INITIALLY THAT DATA ARE GIVEN INDEPENDENT OF TEMPERATURE:     
!  I.E. THAT NTEMP IS ORIGINALLY SET = 1                                
!                                                                       
      NTEMP = 1                                                          
!                                                                       
!  READ THE KEYWORD 'KEY' AND ASSOCIATED PARAMETERS                     
!                                                                       
      ipcount=1
   50 CONTINUE          
      Read (Atom%ColStr(ipcount),100) KEY
      ipcount=ipcount+1
      If (ipcount-1 .gt. Atom%NColStrings) Goto 301
  100 FORMAT  (A)                                                        
      CALL LJUST (KEY)                                                   
      IF (KEY (1:1) .EQ.' ') GOTO 50                                     
      IF (KEY (1:3) .EQ.'END') THEN                                      
         GOTO 300                                                        
      ELSEIF (KEY (1:4) .EQ.'TEMP') THEN                                 
         Read (Atom%ColStr(ipcount), *) NTEMP, (TGRID (JT) , JT = 1, MIN (NTEMP,      &
         MTGRD) )                                                       
         ipcount=ipcount+1
         IF (NTEMP.GT.MTGRD) THEN                                        
            Print *,' GENCOL: WORK ARRAYS (TGRID) TOO SMALL'
            Stop
         ENDIF                                                           
         GOTO 50                                                         
      ELSEIF (KEY (1:4) .EQ.'SEMI') THEN                                 
         Read (Atom%ColStr(ipcount), *) IL, IH, FAB
         ipcount=ipcount+1
      ELSEIF (KEY (1:4) .EQ.'LTDR') THEN                                 
         Read (Atom%ColStr(ipcount), *) IL, IH, ALTDR, BLTDR, CLTDR, DLTDR, FLTDR      
         ipcount=ipcount+1
      ELSEIF (KEY (1:7) .EQ.'SHULL82') THEN                              
         Read (Atom%ColStr(ipcount), *)IL, IH, ACOLSH, TCOLSH, ARADSH, XRADSH, ADISH,&
         BDISH, T0SH, T1SH                                              
         ipcount=ipcount+1
      ELSEIF (KEY (1:3) .EQ.'OHM'.OR.KEY (1:2) .EQ.'CE'.OR.KEY (1:2) .EQ&
     &.'CP'.OR.KEY (1:2) .EQ.'CH'.OR.KEY (1:4) .EQ.'CALP'.OR.KEY (1:4) .&
     &EQ.'RECO'.OR.KEY (1:3) .EQ.'CH0'.OR.KEY (1:3) .EQ.'CH+'.OR.KEY (1:&
     &2) .EQ.'CI') THEN                                                 
         Read (Atom%ColStr(ipcount),*) IL, IH, (CGRID (JT) , JT = 1, NTEMP)        
         ipcount=ipcount+1
         NINTEP = NTEMP                                                  
!                                                                       
!  23-FEB-1994 P.G.JUDGE MODIFICATIONS START:                           
!  SPECIAL CASE OF RAPID TE DEPENDENCE- INTERPOLATE LINEARLY            
         IF (KEY (1:3) .EQ.'CH+'.OR.KEY (1:3) .EQ.'CH0') THEN            
            NINTEP = 2                                                   
            DO 333 JT = 1, NTEMP                                         
               CGRID (JT) = LOG10 (CGRID (JT) )                          
  333       END DO                                                       
         ENDIF                                                           
!                                                                       
!  23-FEB-1994 P.G.JUDGE MODIFICATIONS END:                             
!                                                                       
      ELSE                                                               
         Print *,'GENCOL: Keyword not supported:',KEY
         Stop
      ENDIF                                                              
!                                                                       
!  IDENTIFY THE UPPER AND LOWER LEVELS:                                 
!                                                                       
      ILO = MIN (IL, IH)                                                 
      IHI = MAX (IL, IH)                                                 
!                                                                       
!  CHECK TO SEE THAT THE LEVELS ARE BETWEEN 1 AND NK                    
!                                                                       
      IF (ILO.LT.1.OR.IHI.GT.NK) then
         Print *,' GENCOL:  LEVEL INDEX OUTSI&
     &DE RANGE (1,NK)'
         Stop
      End if
!                                                                       
!  CHECK TO SEE IF THERE ARE LEVELS WITH ENERGIES NOT INCREASING WITH   
!  THE LEVEL LABEL (THIS AFFECTS CI ETC.)                               
!                                                                       
      IF (ATOM%EV (IHI) .LT.ATOM%EV (ILO) ) THEN                                   
         Print *,'Stop in GENCOL'
         Stop
      ENDIF                                                              
!                                                                       
!  DEPTH DO-LOOP:                                                       
!                                                                       
      DO 3 K = 1, NDEP   
!                                                                       
!  SEMI-EMPIRICAL COLLISIONS                                            
!                                                                       
         IF (KEY (1:4) .EQ.'SEMI') THEN                                  
            EUPCM = Atom%EV (IHI) * EE / CC / HH                              
            ELOCM = Atom%EV (ILO) * EE / CC / HH                              
            FEM = FAB * Atom%G (ILO) / Atom%G (IHI)                                
            CT = SEMIC (Atom%ION (ILO) , EUPCM, ELOCM, FEM, TEMP (K) , III)   
            CDN = CT * NE (K)                                            
            CUP = CDN * Nlte%NSTAR (IHI, K) / Nlte%NSTAR (ILO, K)                  
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  CORONAL ION BALANCE DATA (FROM DATA READ IN OPACITY PACKAGE)         
!                                                                       
         ELSEIF (KEY (1:6) .EQ.'CORONA') THEN                            
            Print *,'Option not implemented in GENCOL:'//key
            Stop
         ELSEIF (KEY (1:7) .EQ.'AR85-CH') THEN                           
!                                                                       
!  TABULAR DATA TO BE INTERPOLATED                                      
!                                                                       
         ELSEIF (KEY (1:3) .EQ.'OHM'.OR.KEY (1:2) .EQ.'CE'.OR.KEY (1:2) .EQ&
              &.'CP'.OR.KEY (1:2) .EQ.'CH'.OR.KEY (1:4) .EQ.'CALP'.OR.KEY (1:4) .&
              &EQ.'RECO'.OR.KEY (1:3) .EQ.'CH0'.OR.KEY (1:3) .EQ.'CH+'.OR.KEY (1:&
              &2) .EQ.'CI') THEN                                                 
            IF (TEMP (K) .LT.TGRID (1) ) THEN                            
               CT = CGRID (1)                                            
            ELSEIF (TEMP (K) .GT.TGRID (NTEMP) ) THEN                    
               CT = CGRID (NTEMP)                                        
            ELSE                                                         
               CT = SPLIN (TEMP (K) , TGRID, CGRID, NTEMP, NINTEP)       
            ENDIF
         ENDIF
         IF (KEY (1:3) .EQ.'CH+'.OR.KEY (1:3) .EQ.'CH0') CT = 10.**CT    
         IF (CT.LT.0.) THEN                                              
            Print *,'NEGATIVE COLLISION RATES ENCOUNTERED'
            Stop
         ENDIF
!                                                                       
!  OMEGAS ARE GIVEN (+VE IONS)                                          
!                                                                       
         IF (KEY (1:3) .EQ.'OHM') THEN                                   
            CDN = 8.63E-06 * CT * NE (K) / (Atom%G(IHI) * SQRT (TEMP (K) ) )
            CUP = CDN * Nlte%NSTAR (IHI, K) / Nlte%NSTAR (ILO, K)                  
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  CE VALUES ARE GIVEN (NEUTRALS)                                       
!                                                                       
         ELSEIF (KEY (1:2) .EQ.'CE') THEN                                
            CDN = NE (K) * CT * Atom%G (ILO) * SQRT (TEMP (K) ) / Atom%G (IHI)   
            CUP = CDN * Nlte%NSTAR (IHI, K) / Nlte%NSTAR (ILO, K)               
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  CP VALUES ARE GIVEN (B-B COLLISIONS WITH PROTONS)                    
!                                                                       
         ELSEIF (KEY (1:2) .EQ.'CP') THEN                                
            CDN = NH (6, K) * CT                                       
            CUP = CDN * Nlte%NSTAR (IHI, K) / Nlte%NSTAR (ILO, K)           
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  CH VALUES ARE GIVEN (COLLISIONS WITH NEUTRAL HYDROGEN)               
!                                                                       
         ELSEIF (KEY (1:3) .EQ.'CH ') THEN                               
            CDN = NH (1, K) * CT                                  
            CUP = CDN * Nlte%NSTAR (IHI, K) / Nlte%NSTAR (ILO, K)                  
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  CI VALUES ARE GIVEN                                                  
!                                                                       
         ELSEIF (KEY (1:2) .EQ.'CI') THEN                                
            DEKT = (ATOM%EV (IHI) - ATOM%EV (ILO) ) * EK / TEMP (K)                
            CUP = NE (K) * CT * EXP ( - DEKT) * SQRT (TEMP (K) )       
            CDN = CUP * Nlte%NSTAR (ILO, K) / Nlte%NSTAR (IHI, K)                  
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  CH OR CH+ VALUES GIVEN:  CHARGE TRANSFER COLLISIONS WITH             
!            HYDROGEN ATOMS/IONS  NB:  ORDERING IMPORTANT HERE          
!  EXAMPLE INPUT:                                                       
!                                                                       
!  CH0                                                                  
!  1  2  1.E-9 1.E-9 1.E-9                                              
!                                                                       
!  THIS ASSUMES THAT THE RATE 1.E-9*NH(1,K) WILL BE ADDED TO THE        
!  COLLISION RATE FROM LEVEL 1 TO LEVEL 2                               
!                                                                       
         ELSEIF (KEY (1:3) .EQ.'CH0') THEN                               
            C (IL, IH, K) = NH (1, K) * CT + C (IL, IH, K)          
         ELSEIF (KEY (1:3) .EQ.'CH+') THEN                               
            C (IL, IH, K) = NH (6, K) * CT + C (IL, IH, K)          
!                                                                       
!  LOW-TEMPERATURE DIELECTRONIC RECOMBINATION COEFFS. ARE GIVEN         
!  FORMULA (9) OF NUSSBAUMER AND STOREY (A+A PAPER II)                  
!                                                                       
         ELSEIF (KEY (1:4) .EQ.'LTDR') THEN                              
            T4LTDR = TEMP (K) / 1.E4                                     
            CDN = 1.E-12 * NE (K) * (ALTDR / T4LTDR + BLTDR + CLTDR *  &
            T4LTDR + DLTDR * T4LTDR * T4LTDR) * EXP ( - FLTDR / T4LTDR) &
            / T4LTDR / SQRT (T4LTDR)                                    
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
!                                                                       
!  ADDITION P. JUDGE 24-JAN-1994: BEGIN                                 
!                                                                       
!  PARAMETRIC FITS TO DATA OF ARNAUD AND ROTHENFLUG 1985,               
!  APJS 60, 425.                                                        
!  CDI IS 5 BY MSHELL ELEMENT ARRAY, CONTAINING PARAMETERS              
!  I, A,B,C,D OF TABLE 1 OF ARNAUD AND ROTHENFLUG                       
         ELSEIF (KEY (1:7) .EQ.'AR85-RR') THEN                           
            CDN = ARADAR * (TEMP (K) / 1.E4) **ETAAR                   
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
         ELSEIF (KEY (1:8) .EQ.'AR85-CDI') THEN                          
            CUP = 0.                                                     
            DO 321 J = 1, NCDI                                           
               XJ = CDI (1, J) * EE / BK / TEMP (K)                      
               FXJ = EXP ( - XJ) * SQRT (XJ) * (CDI (2, J) + CDI (3, J) &
               * (1. + XJ) + (CDI (4, J) - XJ * (CDI (2, J) + CDI (3, J)&
               * (2. + XJ) ) ) * FONE (XJ) + CDI (5, J) * XJ * FTWO (XJ)&
               )                                                        
               CUP = CUP + 6.69E-07 / CDI (1, J) / SQRT (CDI (1, J) )   &
               * FXJ                                                    
  321       END DO                                                       
            IF (CUP.LT.0.) then
               print *,'GENCOL: CDI-CUP .LT.0'
               Stop
            End if

            C (ILO, IHI, K) = CUP * NE (K) + C (ILO, IHI, K)            
            C (IHI, ILO, K) = CUP * NE (K) * Nlte%NSTAR (ILO, K) / Nlte%NSTAR (  &
            IHI, K) + C (IHI, ILO, K)                                   
!                                                                       
!  08-MAR-1994 P.G.JUDGE MODIFICATIONS END:                             
!                                                                       
!                                                                       
!  COMPUTATIONS FROM TABLES OF SHULL & VAN STEENBERG,                   
!  AP J SUPPL  48 P95                                                   
!                                                                       
         ELSEIF (KEY (1:7) .EQ.'SHULL82') THEN                           
!                                                                       
!  NOVEMBER 9, 1994, MODIFICATIONS P. JUDGE, BEGIN                      
!  IDEA IS TO REDUCE DIELECTRONIC RECOMBINATION RATE                    
!  OWING TO COLLISIONAL IONIZATION OF RYDBERG STATES                    
!  FOLLOWING SUMMERS 1974, APPLETON LAB REPT.                           
!                                                                       
            SUMMRS = 1.0                                                 
! TO REMOVE DENSITY DEPENDENCE IN DIELECTRONIC RATE COMMENT OUT         
! THE NEXT LINE                                                         
            SUMMRS = SUMMERS (Atom%element, Atom%ION (ILO), ILO, IHI, NE (K) )                         
            CDN = ARADSH * (TEMP (K) / 1.E4) ** ( - XRADSH) + SUMMRS *  &
            ADISH / TEMP (K) / SQRT (TEMP (K) ) * EXP ( - T0SH / TEMP ( &
            K) ) * (1.E0 + BDISH * (EXP ( - T1SH / TEMP (K) ) ) )       
!                                                                       
!  NOVEMBER 9, 1994, MODIFICATIONS P. JUDGE, END                        
!                                                                       
            CDN = CDN * NE (K)                                          
            CUP = ACOLSH * SQRT (TEMP (K) ) * EXP ( - TCOLSH / TEMP (K) &
            ) / (1.E0 + 0.1 * TEMP (K) / TCOLSH)                        
            CUP = CUP * NE (K)                                          
!*                                                                      
!*  3-BODY RECOMBINATION (HIGH DENSITY LIMIT)                           
!*                                                                      
            CDN = CDN + CUP * Nlte%NSTAR (ILO, K) / Nlte%NSTAR (IHI, K)            
!*                                                                      
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  CALP VALUES ARE GIVEN (COLLISIONS WITH ALPHA PARTICLES)              
!  ATOM HAS TO BE HE                                                    
!  INITIA IS CALLED FOR FIRST RATE TO READ POPULATIONS                  
!  FROM RSTRT                                                           
!                                                                       
         ELSEIF (KEY (1:4) .EQ.'CALP') THEN                              
            IF (Atom%element (1:2) .NE.'HE') THEN                              
               Print *,'GENCOL: ATOM HAS TO BE HE FOR CALPHA'        
               Stop
            ENDIF                                                        
            IF (.NOT.ALPRD) THEN                                         
               Print *,'GENCOL: Not supported'
               Stop
!               CALL RDALP                                                
               ALPRD = .TRUE.                                            
            ENDIF                                                        
            CDN = Nlte%NStar (NK, K) * CT                                        
            CUP = CDN * Nlte%NSTAR (IHI, K) / Nlte%NSTAR (ILO, K)                 
            C (IHI, ILO, K) = CDN + C (IHI, ILO, K)                      
            C (ILO, IHI, K) = CUP + C (ILO, IHI, K)                      
!                                                                       
!  RECOMBINATION COEFFICIENTS ARE GIVEN                                 
!  C(J-I)=NE*CT                                                         
!                                                                       
         ELSEIF (KEY (1:4) .EQ.'RECO') THEN                              
            C (IHI, ILO, K) = NE (K) * CT + C (IHI, ILO, K)            
!                                                                       
!  MORE KEYWORDS CAN BE ADDED HERE                                      
!                                                                       
         ENDIF                                                           

3        END DO
      GOTO 50                                                            
!                                                                       
  301 Print *, ' GENCOL: KEYWORD ''END'' NOT FOUND IN COLLISION FILE'
      Stop

  300 CONTINUE                                                           
!
 NLTE%C=C
 Deallocate (C)
 Return
End Subroutine GenCol
!
SUBROUTINE LJUST (TEXT)                                            
!                                                                       
!  LEFT JUSTIFIES THE STRING TEXT                                       
!                                                                       
      CHARACTER ( * ) TEXT                                               
!                                                                       
      L = LEN (TEXT)                                                     
      DO 100 J = 1, L                                                    
         IF (TEXT (J:J) .NE.' ') GOTO 200                                
  100 END DO                                                             
  200 CONTINUE                                                           
      DO 300 I = 1, L                                                    
         IF (J.LE.L) THEN                                                
            TEXT (I:I) = TEXT (J:J)                                      
         ELSE                                                            
            TEXT (I:I) = ' '                                             
         ENDIF                                                           
         J = J + 1                                                       
  300 END DO                                                             
!                                                                       
      RETURN                                                             
    END SUBROUTINE LJUST
!                                                                       
!***********************************************************************
!                                                                       
      FUNCTION FONE (X)                                                  
!                                                                       
! NEW ROUTINE 24-JAN-1994                                               
! CALCULATES F1(X) NEEDED FOR COLLISIONAL RATES OF ARNAUD AND ROTHENFLUG
! MODIFIED BY P. JUDGE 15-MAR-1994                                      
! FOR LARGE VALUES OF X, USE ASYMPTOTIC LIMIT                           
!                                                                       
      DATA SPLIT / 50.0 /                                                
      IF (X.LE.SPLIT) THEN                                               
         FONE = EXP (X) * EXPINT (1, X, EX)                              
      ELSE                                                               
         FONE = 1. / X                                                   
      ENDIF                                                              
      RETURN                                                             
      END FUNCTION FONE                             
!                                                                       
!********************************************************************** 
!                                                                       
      FUNCTION FTWO (X)                                                  
!                                                                       
! NEW ROUTINE 24-JAN-1994  P. JUDGE                                     
! CALCULATES F2(X) NEEDED FOR COLLISIONAL RATES OF ARNAUD AND ROTHENFLUG
! IF ARGUMENT X IS < BREAK, THEN USE THE EQNS 5 AND 6 OF HUMMER (1983, J
! BREAK GIVEN BY HUMMER AS 4.0                                          
!                                                                       
! NOTE THAT THE SUGGESTED POLYNOMIAL EXPANSION OF ARNAUD AND ROTHENFLUG 
! FAILS FOR ARGUMENTS < BREAK- SEE HUMMER'S ORIGINAL PAPER.             
! IT IS UNCLEAR IF THE TABULATED IONIZATION FRACTIONS OF ARNAUD AND ROTH
! CONTAIN THIS ERROR.                                                   
!                                                                       
      DIMENSION P (15) , Q (15)                                          
      DATA BREAK / 4.0 /                                                 
      DATA P / 1.0, 2.1658E+02, 2.0336E+04, 1.0911E+06, 3.7114E+07,     &
      8.3963E+08, 1.2889E+10, 1.3449E+11, 9.4002E+11, 4.2571E+12,       &
      1.1743E+13, 1.7549E+13, 1.0806E+13, 4.9776E+11, 0.0000 / , Q /    &
      1.0, 2.1958E+02, 2.0984E+04, 1.1517E+06, 4.0349E+07, 9.4900E+08,  &
      1.5345E+10, 1.7182E+11, 1.3249E+12, 6.9071E+12, 2.3531E+13,       &
      4.9432E+13, 5.7760E+13, 3.0225E+13, 3.3641E+12 /                  
      IF (X.GE.BREAK) THEN                                               
         PX = P (1)                                                      
         XFACT = 1.0                                                     
         DO 1 I = 2, 15                                                  
            XFACT = XFACT / X                                            
            PX = PX + P (I) * XFACT                                      
    1    END DO                                                          
!        PX = P(1) + XI*(P(2) + (XI*P(3) +(XI*P(4) + (XI*P(5) + (XI*P(6)
!     *   + (XI*P(7) + (XI*P(8) +(XI*P(9) + (XI*P(10) + (XI*P(11) +     
!     *   (XI*P(12) + (XI*P(13) +(XI*P(14)+XI*P(15))))))))))))))        
!        QX = Q(1) + XI*(Q(2) + (XI*Q(3) +(XI*Q(4) + (XI*Q(5) + (XI*Q(6)
!     *   + (XI*Q(7) + (XI*Q(8) +(XI*Q(9) + (XI*Q(10) + (XI*Q(11) +     
!     *   (XI*Q(12) + (XI*Q(13) +(XI*Q(14)+XI*Q(15))))))))))))))        
         QX = Q (1)                                                      
         XFACT = 1.0                                                     
         DO 2 I = 2, 15                                                  
            XFACT = XFACT / X                                            
            QX = QX + Q (I) * XFACT                                      
    2    END DO                                                          
         FTWO = PX / QX / X / X                                          
      ELSE                                                               
!                                                                       
!  HUMMER'S EQUNS 5 AND 6                                               
!  GAMMA IS EULER'S CONSTANT (ABRAMOVICZ+STEGUN)                        
!                                                                       
         GAMMA = 0.5772156649                                            
         F0X = PI * PI / 12.                                             
         TERM = 1.0                                                      
         COUNT = 0.0                                                     
         FACT = 1.0                                                      
         XFACT = 1.0                                                     
         DO WHILE (ABS (TERM / F0X) .GT.1.E-8)                           
         COUNT = COUNT + 1.0                                             
         FACT = FACT * COUNT                                             
         XFACT = XFACT * ( - X)                                          
         TERM = XFACT / COUNT / COUNT / FACT                             
         F0X = F0X + TERM                                                
         IF (COUNT.GT.100.) then
            print *,'FTWO: TOO MANY ITERATIONS'
            Stop
         End if
         ENDDO                                                           
         FTWO = EXP (X) * ( (LOG (X) + GAMMA) * (LOG (X) + GAMMA)       &
         * 0.5 + F0X)                                                   
      ENDIF                                                              
      RETURN                                                             
      END FUNCTION FTWO                             
                                      

     FUNCTION SUMMERS (ARELEM, ICHRGE, ILO, IHI, EDENS)                                 
!                                                                       
!  NEW ROUTINE FOR COMPUTING COLLISIONAL REDUCTION OF DIELECTRONIC      
!  RECOMBINATION RATE BY ELECTRON COLLISIONS FOLLOWING SUMMERS 1974     
!  Inputs:                                                              
!    ILO  - index of lower level of transition                          
!    IHI  - index of upper level of transition                          
!    EDENS - electron density in /cm3                                   
! SUMMERS 94-11-09  NEW ROUTINE: (PHILIP JUDGE)                         
!                                                                       
      CHARACTER(2) CSEQ, ARELEM, ATOMNM                                  
      LOGICAL DEBUG                                                      
      DATA DEBUG / .FALSE. /                                             
      SUMMERS = 1.0                                                      
!                                                                       
!                                                                       
!  FIND ELEMENT                                                         
!                                                                       
      IZ = IATOMN (ARELEM)                                               
      ZZ = 0.0 + IZ                                                      
!                                                                       
!  FIND ISO-ELECTRONIC SEQUENCE OF RECOMBINING ION                      
!                                                                       
      ZZ = FLOAT (ICHRGE)                                                
      ISOSEQ = IZ - ICHRGE                                               
      CSEQ = ATOMNM (ISOSEQ)                                             
!                                                                       
!*********************************************************************  
! LITHIUM, SODIUM, POTASSIUM SEQUENCES                                  
!*********************************************************************  
!                                                                       
      RHO0 = 2000.0                                                      
      IF (CSEQ.EQ.'LI'.OR.CSEQ.EQ.'NA'.OR.CSEQ.EQ.'K') RHO0 = 30.0       
      RHOQ = EDENS / ZZ / ZZ / ZZ / ZZ / ZZ / ZZ / ZZ                    
      SUMMERS = 1. / (1. + RHOQ / RHO0) **0.14                           

      RETURN                                                             
      END FUNCTION SUMMERS            

!                                                                       
! ****************************************************************      
!                                                                       
      FUNCTION SPLIN (T, X, Y, N, NNN)                                   
!                                                                       
!  SPLINE INTERPOLATION OF T IN A GIVEN TABLE OF POINTS(X(I),Y(I)).     
!                                                                       
! Y(I) IS A FUNCTION OF X(I) - X(N)>X(N-1)>....X(1) -OR- X(1)> X(2)     
! > ....> X(N) - WHEN X IS NORMALISED X(MIN) = 0, XMAX=1,               
! NORM(ABS(X(I+1)-X(I))) < 1.0E-8,(I=1,N-1). THE ARRAY X IS DIVIDED     
! INTO OVERLAPPING INTERVALS. EACH INTERVAL CONSISTS OF NNN POINTS,     
! THE OVERLAP IS INT(LOG(NNN)+3) POINTS. ( 3<=NNN<=100,N).              
! EACH TIME AFTER A CALL WHEN T CHANGES OF INTERVAL OR AFTER A CALL     
! WITH ANOTHER SET OF POINTS, A NEW CUBIC SPLINE OF NNN POINTS HAS      
! TO BE COMPUTED, THEREFORE THE COMPUTING TIME DEPENDS STRONGLY OF      
! THE SUCCESSION OF CALLS.                                              
! FOR EXAMPLE: SUCCESSIVE CALLS, WHERE T IS RANDOMLY CHOSEN WILL COST   
! A LOT OF COMPUTING TIME - (T OFTEN CHANGES OF INTERVAL) --->          
! TAKE NNN THEN AS LARGE AS POSSIBLE. SUCCESSIVE CALLS WHERE T CHANGES  
! SLIGHTLY COST LESS COMPUTING TIME, SO IT WILL BE CLEAR THAT YOUR      
! CHOICE OF NNN DEPENDS HEAVILY UPON ITS WAY OF USEAGE.                 
!                                                                       
! COMPUTING TIME:   FOR THE COMPUTATION OF SPLINE COEFFICIENTS          
!        THIS ROUTINE NEEDS ABOUT 21*N MULTIPLICATIONS OR DIVISIONS     
!        FOR THE INTERPOLATION BY MEANS OF THE SPLINE COEFFICIENTS      
!        THE ROUTINE NEEDS 9 MULTIPLICATIONS OR DIVISIONS.              
!                                                                       
! EXTRAPOLATION: IF T IS NOT IN [X(1),X(N)] T IS LINEAR EXTRAPOLATED    
!                                                                       
! A PART OF THE ALGORITM FOR THE COMPUTATION OF A NATURAL SPLINE        
! IS TAKEN FROM T.N.E. GREVILLE, THE LINEAR SYSTEM IS SOLVED BY         
! THE METHOD OF SUCCESSIVE OVERRELAXATION. (ERROR = 1.0E-6)             
!                                                                       
! REF: T.N.E. GREVILLE, MATHEMATICS RESEARCH CENTER, U.S. ARMY,         
!        UNIVERSITY OF WISCONSIN. MATHEMATICAL METHODS.                 
!                                                                       
! THIS ROUTINE WAS IMPLEMENTED BY E.B.J. VAN DER ZALM, STERRENWACHT     
! UTRECHT (MAY 7TH 1981). ADAPTED FOR VAX\11 BY PAUL KUIN AT OXFORD.    
! WRITTEN IN REAL*4 BY P.G. JUDGE, OXFORD                               
! MODIFIED SO THAT IF THE ARGUMENT IS OUTSIDE THE RANGE OF X-VALUES     
! THEN THE FIRST (OR LAST) Y-VALUE IS TAKEN                             
!                                                                       
! INPUT:         T        ARGUMENT   (REAL)                             
!                X        ARRAY OF ARGUMENTS        (REAL)              
!                Y        ARRAY OF FUNCTION VALUES (REAL)               
!                N        LENGTH OF THE ARRAYS X,Y                      
!                        IF N = 2 ---> LINEAR INTERPOLATION             
!                NNN        NUMBER OF POINTS OF THE CUBIC SPLINE        
!                        IF NNN > N   ---> NNN = N                      
!                        IF NNN > 100 ---> NNN = 100                    
!                        IF NNN = 2 LINEAR INTERPOLATION ADOPTED.       
!                                                                       
!                        IF NNN IS ZERO NNN=7  (ALTERED BY P JUDGE)     
!                       (USED TO BE IF NNN OMITTED )                    
! OUTPUT:        SPLIN INTERPOLATED VALUE AT T                          
!                                                                       
      DIMENSION X (N) , Y (N) , S2 (100) , S3 (100) , DELY (100) , H (  &
      100) , H2 (100) , DELSQY (100) , C (100) , B (100) , XX (100) ,   &
      YY (100)                                                          
      LOGICAL SEARCH, FOUND                                              
      CHARACTER(4) INIT                                                  
      DATA INIT / 'NOTO' /                                               
      SAVE                                                               
!                                                                       
!  NN = NUMBER OF POINTS OF THE SPLINE INTERVAL                         
!                                                                       
      NN = MIN (7, N)                                                    
      IF (NNN.EQ.0) GOTO 10                                              
! PGJ        IF (%LOC(NNN).EQ.0) GO TO 10                               
      NN = NNN                                                           
      NN = MIN (N, NNN, 100)                                             
!                                                                       
!           IF T IN [X(K)-ERROR,X(K)+ERROR] T = X(K)                    
!                                                                       
   10 ERROR = 1.0E-12                                                    
      IF (T.NE.0) ERROR = ABS (1.0E-12 * T)                              
      FOUND = SEARCH (X, T, 1, N, K, ERROR)                              
      IF (.NOT.FOUND) GOTO 20                                            
      SPLIN = Y (K)                                                      
      RETURN                                                             
   20 IF (N.NE.2.AND.NN.NE.2) GOTO 30                                    
!                                                                       
!           IF N=2 OR NNN=2 LINEAR INTERPOLATION IS ADOPTED             
!                                                                       
      SPLIN = (Y (K + 1) - Y (K) ) * (T - X (K) ) / (X (K + 1) - X (K) )&
      + Y (K)                                                           
      RETURN                                                             
   30 IF (N.NE.1) GOTO 40                                                
      SPLIN = Y (1)                                                      
      RETURN                                                             
!                                                                       
!           INTERVAL OF THE SPLINE IS COMPUTED                          
!                                                                       
   40 IOVER = LOG10 (FLOAT (NN) ) * 3                                    
      INT = (K - IOVER) / (NN - 2 * IOVER) + 1                           
      I1 = (INT - 1) * (NN - 2 * IOVER)                                  
      I2 = I1 + NN - 1                                                   
      IF (I1.GE.1) GOTO 50                                               
      I2 = NN                                                            
      I1 = 1                                                             
   50 IF (I2.LE.N) GOTO 60                                               
      I2 = N                                                             
      I1 = N - NN + 1                                                    
   60 ISHIFT = I1 - 1                                                    
      N1 = NN - 1                                                        
!                                                                       
!           INITIALIZATION CHECK                                        
!                                                                       
      IF (INIT.NE.'OKAY') GOTO 80                                        
!                                                                       
!           INTERVAL CHECK                                              
!                                                                       
      DO 70 I = 1, NN                                                    
         IF (XX (I) .NE.X (I + ISHIFT) ) GOTO 90                         
         IF (YY (I) .NE.Y (I + ISHIFT) ) GOTO 90                         
   70 END DO                                                             
      GOTO 200                                                           
   80 INIT = 'OKAY'                                                      
!                                                                       
!           START OF THE COMPUTATION OF SPLINE COEFFICIENTS             
!                                                                       
   90 RMIN = 1.0E37                                                      
      RMAX = - 1.0E37                                                    
      DO 100 I = 1, NN                                                   
         IF (Y (I + ISHIFT) .LT.RMIN) RMIN = Y (I + ISHIFT)              
         IF (Y (I + ISHIFT) .GT.RMAX) RMAX = Y (I + ISHIFT)              
  100 END DO                                                             
!                                                                       
!           COMPUTATION OF THE NORM FACTORS XNORM AND YNORM             
!                                                                       
      XNORM = 1 / (X (NN + ISHIFT) - X (1 + ISHIFT) )                    
      YNORM = RMAX - RMIN                                                
      IF (YNORM.EQ.0) YNORM = RMAX                                       
      IF (YNORM.EQ.0) YNORM = 1                                          
      YNORM = 1 / YNORM                                                  
      DO 110 I = 1, N1                                                   
         H (I) = (X (I + 1 + ISHIFT) - X (I + ISHIFT) ) * XNORM          
         DELY (I) = ( (Y (I + 1 + ISHIFT) - Y (I + ISHIFT) ) / H (I) )  &
         * YNORM                                                        
  110 END DO                                                             
      DO 120 I = 2, N1                                                   
         H2 (I) = H (I - 1) + H (I)                                      
         B (I) = 0.5 * H (I - 1) / H2 (I)                                
         DELSQY (I) = (DELY (I) - DELY (I - 1) ) / H2 (I)                
         S2 (I) = 2. * DELSQY (I)                                        
         C (I) = 3. * DELSQY (I)                                         
  120 END DO                                                             
      S2 (1) = 0                                                         
      S2 (N1 + 1) = 0                                                    
!                                                                       
!        SOLUTION OF THE LINEAR SYSTEM OF THE SPLINE COEFFICIENTS       
!        BY SUCCESSIVE OVERRELAXATION.                                  
!        CONSTANTS: EPS = ERROR CRITERION IN THE ITERATIVE SOLUTION.    
!                   OMEGA = RELAXATION COEFFICIENT.                     
!                                                                       
      EPSLN = 1.0E-8                                                     
      OMEGA = 1.0717968                                                  
  130 ETA = 0.                                                           
      DO 160 I = 2, N1                                                   
         W = (C (I) - B (I) * S2 (I - 1) - (0.5 - B (I) ) * S2 (I + 1)  &
         - S2 (I) ) * OMEGA                                             
         IF (ABS (W) - ETA) 150, 150, 140                                
  140    ETA = ABS (W)                                                   
  150    S2 (I) = S2 (I) + W                                             
  160 END DO                                                             
      IF (ETA - EPSLN) 170, 130, 130                                     
  170 DO 180 I = 1, N1                                                   
         S3 (I) = (S2 (I + 1) - S2 (I) ) / H (I)                         
  180 END DO                                                             
!                                                                       
!        X AND Y STORED IN XX AND YY FOR INTERVAL CHECK                 
!                                                                       
      DO 190 I = 1, NN                                                   
         XX (I) = X (I + ISHIFT)                                         
         YY (I) = Y (I + ISHIFT)                                         
  190 END DO                                                             
  200 I = K - I1 + 1                                                     
      HT1 = (T - X (I + ISHIFT) ) * XNORM                                
      HT2 = (T - X (I + ISHIFT + 1) ) * XNORM                            
      IF ( (T - X (1) ) * XNORM.GT.0) GOTO 210                           
!                                                                       
!        EXTRAPOLATION   T < X(1)                                       
!                                                                       
      SPLIN = Y (1)                                                      
      RETURN                                                             
  210 IF ( (T - X (N) ) * XNORM.GT.0) GOTO 220                           
!                                                                       
!        INTERPOLATION BY MEANS OF THE SPLINE COEFFICIENTS              
!                                                                       
      SS2 = S2 (I) + HT1 * S3 (I)                                        
      PROD = HT1 * HT2                                                   
      DELSQS = (S2 (I) + S2 (I + 1) + SS2) / 6.                          
      SPLIN = Y (I + ISHIFT) + (HT1 * DELY (I) + PROD * DELSQS) / YNORM  
      RETURN                                                             
!                                                                       
!        EXTRAPOLATION   T > X(N)                                       
!                                                                       
  220 SPLIN = Y (N)                                                      
      RETURN                                                             
      END FUNCTION SPLIN            
!                                                                       
!***********************************************************************
!                                                                       
      FUNCTION SEMIC (Z, EUP, ELO, FEM, TEMP, IFLAG)                     
!                                                                       
!  RETURNS COLLISION RATE DOWNWARDS USING SEMI-EMPIRICAL GF'S           
!                                                                       
!  P.G. JUDGE, NOVEMBER 1987.                                           
!                                                                       
!  INPUT:                                                               
!         Z      - CHARGE OF ION+1 (E.G. FOR C II Z=2)                  
!         EUP    - ENERGY OF UPPER LEVEL IN CM-1                        
!         ELO    - ENERGY OF LOWER LEVEL IN CM-1                        
!         FEM    - EMISSION OSCILLATOR STRENGTH ( = GF/G(UPPER))        
!         TEMP   - TEMPERATURE IN DEGREES KELVIN                        
!                                                                       
!  OUTPUT:                                                              
!         IFLAG  -  = 1 FOR VAN REGEMORTER APPROXIMATION                
!         IFLAG  -  = 2 FOR SHEVELKO       APPROXIMATION                
!         SEMIC-  DOWNWARD COLLISION RATE IN CM3 /S                     
!                                                                       
      INTEGER Z                                                          
      DATA C1, RY / 1.43882, 109737.312 /                                
!                                                                       
! INITIALIZE                                                            
!                                                                       
      DELTE = EUP - ELO                                                  
      BETA = DELTE * C1 / TEMP                                           
      SEMIC = 0.0                                                        
!                                                                       
!  IFLAG=1 => VAN REGEMORTER APPROXN.                                   
!                                                                       
      IF (BETA.GE.0.01) THEN                                             
         IFLAG = 1                                                       
         SEMIC = 3.2E-7 * FEM * (RY / DELTE) **1.5 * SQRT (BETA)        &
         * PSEMI (Z, BETA)                                              
!                                                                       
!  IFLAG=2 => SHEVELKO APPROX                                           
!                                                                       
      ELSEIF (BETA.LT.0.01) THEN                                         
         IFLAG = 2                                                       
         SEMIC = 1.74E-07 * FEM * EXP (BETA) / SQRT (BETA * DELTE / RY) &
         * LOG (ELO / DELTE * SQRT (RY / C1 / TEMP) )                   
      ENDIF                                                              
!                                                                       
      RETURN                                                             
      END FUNCTION SEMIC   
!                                                                       
! **********************************************************************
!                                                                       
      FUNCTION PSEMI (Z, B)                                              
!                                                                       
!  THERMAL P-FUNCTION FOR SEMIC EMPIRICAL COLLISION RATES               
!  REFERENCE: SOBEL'MAN - 'ATOMIC PHYSICS II.  '                        
!                                                                       
!:                                                                      
!: PSEMI  94-08-29  MODIFICATIONS: (MATS CARLSSON)                      
!:        MULTIPLE RETURN CHANGED TO SINGLE RETURN TO AVOID             
!:        COMPILER BUG IN DIGITAL FORTRAN                               
!:                                                                      
      PARAMETER (N1 = 10)                                                
      DIMENSION BETREF (N1) , PNREF (N1) , PCREF (N1)                    
      INTEGER Z                                                          
      DATA BETREF / - 2.0, - 1.699, - 1.398, - 1.0, - 0.699, - 0.398,   &
      0.0, 0.301, 0.602, 1.0 /                                          
      DATA PNREF / 1.160, 0.956, 0.758, 0.493, 0.331, 0.209, 0.100,     &
      0.063, 0.040, 0.023 /                                             
      DATA PCREF / 1.160, 0.977, 0.788, 0.554, 0.403, 0.290, 0.214,     &
      0.201, 0.200, 0.200 /                                             
!                                                                       
!  LIMIT OF HIGH TEMPERATURE: KT >> E                                   
!                                                                       
      IF (B.LT.0.01) THEN                                                
         PSEMI = 0.27566 * (0.577 + LOG (B) )                            
!                                                                       
!  INTERMEDIATE TEMPS (MOST IMPORTANT FOR EQM PLASMAS)                  
!  SPLINE INTERPOLATION ONTO LOGB GRID:                                 
!                                                                       
      ELSEIF (B.GT.0.01.AND.B.LT.10.0) THEN                              
         BB = LOG10 (B)                                                  
         IF (Z.EQ.1) PSEMI = SPLIN (BB, BETREF, PNREF, N1, N1)           
         IF (Z.GT.1) PSEMI = SPLIN (BB, BETREF, PCREF, N1, N1)           
!                                                                       
!  LIMIT OF LOW TEMPERATURE: KT << E                                    
!                                                                       
      ELSEIF (B.GT.10.0) THEN                                            
         IF (Z.EQ.1) PSEMI = 0.066 / SQRT (B)                            
         IF (Z.GT.1) PSEMI = 0.200                                       
      ENDIF                                                              
      RETURN                                                             
      END FUNCTION PSEMI        
!
      !                                                                       
! **********************************************************************
!                                                                       
      LOGICAL FUNCTION SEARCH (ARR, X, IA, IB, K, ERR)                   
!                                                                       
! * SEARCHES THE POINT X WITHIN AN ERROR BOUND -ERR- IN ARRAY ARR      *
! * IF THAT POINT IS NOT FOUND, IT GIVES THE                           *
! * INTERVAL IN THE ARRAY, WHERE THAT POINT FITS.                      *
! * THE INTERVAL K IF X IN [ARR(K),ARR(K+1)), INTERVAL 1 IF X IN       *
! * (-INF,ARR(2)), INTERVAL N-1 IF X IN [ARR(N-1),INF)-(ARR INCREASING)*
! * INTERVAL 1 IF X IN (INF,ARR(2)),INTERVAL N-1 IF X IN [ARR(N-1),    *
! * -INF)-(ARR DECREASING)                                             *
! * IN THE WORST CASE A SEARCH COSTS 2*LOG2(N) CYCLES.                 *
! * THIS ROUTINE IS VERY ECONOMIC, WHEN IN A SEQUENCE OF SEARCHES,     *
! * THE DIFFERENCES BETWEEN THE POINTS WHICH ARE TO BE SEARCHED ARE    *
! * SMALL.                                                             *
! * FOR EXAMPLE: A SEQUENCE OF SEARCHES WHICH GOES ALONG THE ARRAY,    *
! * OR A SEQUENCE OF SEARCHES WHICH REMAINS IN A SMALL AREA OF         *
! * THE ARRAY.                                                         *
! *                                                                    *
! * INPUT:     - ARR         ARRAY WITH A NON DECREASING OR A NON      *
! *                          INCREASING FUCNTION.                      *
! *            - IA,IB       LOWER AND UPPER LIMIT OF THE ARRAY WITHIN *
! *                          IS SEARCHED.                              *
! *            - X           VALUE WHICH YOU WANT TO SEARCH            *
! *            - K           LAST INDEX WHICH WAS FOUND                *
! *                          SEARCH                                    *
! *            - ERR         ERROR BOUND                               *
! *                                                                    *
! * OUTPUT:    - K           INDEX OF THE POINT OR INTERVAL OF THE     *
! *                          ARRAY WHICH IS FOUND.                     *
! *            - SEARCH      TRUE - X IS FOUND WITHIN THE ERRORBOUND   *
! *                                  -ERR-                             *
! *                          FALSE - X IS NOT FOUND.                   *
! *                                                                    *
! * SEND IN BY E.V.D.ZALM, UTRECHT, STERREWACHT, MARCH 24TH 1981       *
! **********************************************************************
      DIMENSION ARR (IB)                                                 
!                                                                       
      ONE = 1.0                                                          
      SEARCH = .TRUE.                                                    
      IF (ARR (IB) - ARR (IA) .LT.0.0) GOTO 100                          
    5 IF (X.LT.ARR (IA) .OR.X.GT.ARR (IB) ) GOTO 25                      
      IF (K.LT.IA.OR.K.GT.IB - 1) GOTO 25                                
      L = K                                                              
      IP = SIGN (ONE, X - ARR (K) )                                      
      IF (IP.LT.0.) GOTO 20                                              
   10 L = MIN (K + IP, IB)                                               
      IF (ARR (L) .GE.X) GOTO 30                                         
      IP = IP * 2                                                        
      K = L                                                              
      GOTO 10                                                            
   20 K = MAX (L + IP, IA)                                               
      IF (ARR (K) .LE.X) GOTO 30                                         
      IP = IP * 2                                                        
      L = K                                                              
      GOTO 20                                                            
   25 K = IA                                                             
      L = IB                                                             
   30 IF ( (L - K) .LE.1) GOTO 50                                        
      I = INT ( (K + L) / 2.)                                            
      IF (ARR (I) .GE.X) GOTO 40                                         
      K = I                                                              
      GOTO 30                                                            
   40 L = I                                                              
      GOTO 30                                                            
  100 IF (X.LT.ARR (IB) .OR.X.GT.ARR (IA) ) GOTO 250                     
      IF (K.LT.IA.OR.K.GT.IB - 1) GOTO 250                               
      L = K                                                              
      IP = SIGN (ONE, ARR (K) - X)                                       
      IF (IP.LT.0.) GOTO 200                                             
  110 L = MIN (K + IP, IB)                                               
      IF (ARR (L) .LE.X) GOTO 300                                        
      IP = IP * 2                                                        
      K = L                                                              
      GOTO 110                                                           
  200 K = MAX (L + IP, IA)                                               
      IF (ARR (K) .GE.X) GOTO 300                                        
      IP = IP * 2                                                        
      L = K                                                              
      GOTO 200                                                           
  250 K = IA                                                             
      L = IB                                                             
  300 IF ( (L - K) .LE.1) GOTO 50                                        
      I = INT ( (K + L) / 2.)                                            
      IF (ARR (I) .LE.X) GOTO 400                                        
      K = I                                                              
      GOTO 300                                                           
  400 L = I                                                              
      GOTO 300                                                           
   50 IF (ABS (X - ARR (K) ) .LE.ERR) RETURN                             
      IF (ABS (X - ARR (L) ) .GT.ERR) GOTO 60                            
      K = L                                                              
      RETURN                                                             
   60 SEARCH = .FALSE.                                                   
      RETURN                                                             
      END FUNCTION SEARCH            
!
!                                                                       
!*********************************************************************  
!                                                                       
      INTEGER FUNCTION IATOMN (STRING)                                   
!                                                                       
!  ATOMN 94-02-22  NEW ROUTINE: (PHILIP JUDGE)                          
!  GIVES ATOMIC NUMBER OF ARELEMENT IF STRING IS A STRING CONTAINING    
!  THE NAME OF THE ARELEMENT, E.G. IF INPUT  IS 'H ' IT WILL RETURN 1   
!                                                                       
      PARAMETER (NDATA = 28)                                             
      CHARACTER(2) ARELEM (NDATA)                                        
      CHARACTER ( * ) STRING                                             
      DATA ARELEM / 'H ', 'HE', 'LI', 'BE', 'B ', 'C ', 'N ', 'O ', 'F '&
     &, 'NE', 'NA', 'MG', 'AL', 'SI', 'P ', 'S ', 'CL', 'AR', 'K ', 'CA'&
     &, 'SC', 'TI', 'V ', 'CR', 'MN', 'FE', 'CO', 'NI' /                
      IATOMN = - 1                                                       
      DO 1 I = 1, NDATA                                                  
         IF (STRING.EQ.ARELEM (I) ) THEN                                 
            IATOMN = I                                                   
            RETURN                                                       
         ENDIF                                                           
    1 END DO                                                             
      END FUNCTION IATOMN                           
!                                                                       
!*********************************************************************  
!                                                                       
      CHARACTER(2) FUNCTION ATOMNM (I)                                   
!                                                                       
!  ATOMNM 94-02-22  NEW ROUTINE: (PHILIP JUDGE)                         
!  GIVES ATOMIC NAME OF ARELEMENT IF I IS AN INTEGER CONTAINING         
!  E.G. IF INPUT  IS 'H ' IT WILL RETURN 1, 'HE', IT WILL RETURN 2, ETC.
!                                                                       
      PARAMETER (NDATA = 28)                                             
      CHARACTER(2) ARELEM (NDATA)                                        
      DATA ARELEM / 'H ', 'HE', 'LI', 'BE', 'B ', 'C ', 'N ', 'O ', 'F '&
     &, 'NE', 'NA', 'MG', 'AL', 'SI', 'P ', 'S ', 'CL', 'AR', 'K ', 'CA'&
     &, 'SC', 'TI', 'V ', 'CR', 'MN', 'FE', 'CO', 'NI' /                
      IF (I.LE.NDATA.AND.I.GE.1) THEN                                    
         ATOMNM = ARELEM (I)                                             
      ELSE                                                               
      ATOMNM = '  '                                                      
      ENDIF                                                              
      RETURN                                                             
      END FUNCTION ATOMNM              
!                                                                       
!*********************************************************************  
!                                                                       
      FUNCTION EXPINT (N, X, EX)                                         
!                                                                       
!  COMPUTES THE N'TH EXPONENTIAL INTEGRAL OF X                          
!  INPUT - X,  INDEPENDENT VARIABLE (-100. .LE. X .LE. +100.)           
!         N,  ORDER OF DESIRED EXPONENTIAL INTEGRAL (1 .LE. N .LE. 8)   
!  OUTPUT - EXPINT,  THE DESIRED RESULT                                 
!           EX,  EXPF(-X)                                               
!  NOTE   RETURNS WITH E1(0)=0, (NOT INFINITY).                         
!  BASED ON THE SHARE ROUTINE NUEXPI, WRITTEN BY J. W. COOLEY,          
!  COURANT INSTITUTE OF MATHEMATICAL SCIENCES, NEW YORK UNIVERSITY      
!  OBTAINED FROM RUDOLF LOESER                                          
!-----GENERAL COMPILATION OF 1 AUGUST 1967.                             
!                                                                       
      DIMENSION TAB (20) , XINT (7)                                      
      DATA XINT / 1., 2., 3., 4., 5., 6., 7. /                           
      DATA TAB / .2707662555, .2131473101, .1746297218, .1477309984,    &
      .1280843565, .1131470205, .1014028126, .0919145454, .0840790292,  &
      .0774922515, .0718735405, .0670215610, .0627878642, .0590604044,  &
      .0557529077, .0527977953, .0501413386, .0477402600, .0455592945,  &
      .0435694088 /                                                     
      DATA XSAVE / 0. /                                                  
!                                                                       
      U = X                                                              
      IF (U) 603, 602, 603                                               
  602 EX = 1.                                                            
      IF (N - 1) 800, 800, 801                                           
  800 EXPINT = 0.                                                        
      GOTO 777                                                           
  801 EXPINT = 1. / XINT (N - 1)                                         
      GOTO 777                                                           
  603 IF (U - XSAVE) 604, 503, 604                                       
  604 XSAVE = U                                                          
      XM = - U                                                           
      EMX = EXP (XM)                                                     
!                                                                       
!  SELECT METHOD FOR COMPUTING EI(XM)                                   
!                                                                       
      IF (XM - 24.5) 501, 400, 400                                       
  501 IF (XM - 5.) 502, 300, 300                                         
  502 IF (XM + 1.) 100, 200, 200                                         
  503 EISAVE = - ARG                                                     
      EXSAVE = EMX                                                       
!                                                                       
!  NOW RECURSE TO HIGHER ORDERS                                         
!                                                                       
      IF (N - 1) 507, 507, 505                                           
  505 DO 506 I = 2, N                                                    
         EISAVE = (U * EISAVE-EXSAVE) / ( - XINT (I - 1) )               
  506 END DO                                                             
  507 EXPINT = EISAVE                                                    
      EX = EXSAVE                                                        
  777 RETURN                                                             
!                                                                       
!  EI(XM) FOR XM .LT. -1.0                                              
!  HASTINGS POLYNOMIAL APPROXIMATION                                    
!                                                                       
  100 ARG = ( ( ( ( ( (U + 8.573328740) * U + 18.05901697) * U +        &
      8.634760893) * U + .2677737343) / XM) * EMX) / ( ( ( (U +         &
      9.573322345) * U + 25.63295615) * U + 21.09965308) * U +          &
      3.958496923)                                                      
      GOTO 503                                                           
!     EI(XM) FOR -1. .LE. XM .LT. 5.0                                   
!     POWER SERIES EXPANSION ABOUT ZERO                                 
  200 ARG = LOG (ABS (XM) )                                              
      ARG = ( ( ( ( ( ( ( ( ( ( ( ( ( ( ( (.41159050E-14 * XM +         &
      .71745406E-13) * XM + .76404637E-12) * XM + .11395905E-10)        &
      * XM + .17540077E-9) * XM + .23002666E-8) * XM + .27536018E-7)    &
      * XM + .30588626E-6) * XM + .31003842E-5) * XM + .28346991E-4)    &
      * XM + .23148057E-3) * XM + .0016666574) * XM + .010416668)       &
      * XM + .055555572) * XM + .25) * XM + .99999999) * XM + .57721566)&
      + ARG                                                             
      GOTO 503                                                           
!                                                                       
!  EI(XM) FOR 5.0 .LE. XM .LT. 24.5                                     
!  TABLE LOOK-UP AND INTERPOLATION                                      
!                                                                       
  300 I = XM + .5                                                        
      XZERO = I                                                          
      DELTA = XZERO - XM                                                 
      ARG = TAB (I - 4)                                                  
      IF (DELTA) 303, 305, 303                                           
  303 Y = ARG                                                            
      DELTAX = DELTA / XZERO                                             
      POWER = 1. / DELTAX                                                
      DO 304 I = 1, 7                                                    
         POWER = POWER * DELTAX                                          
         Y = ( (Y - POWER / XZERO) * DELTA) / XINT (I)                   
         ARG = ARG + Y                                                   
         IF (ABS (Y / ARG) - 1.E-8) 305, 304, 304                        
  304 END DO                                                             
  305 ARG = EMX * ARG                                                    
      GOTO 503                                                           
!     EI(XM) FOR 24.5 .LE. XM                                           
!     TRUNCATED CONTINUED FRACTION                                      
  400 ARG = ( ( ( (XM - 15.) * XM + 58.) * XM - 50.) * EMX) / ( ( ( (XM &
      - 16.) * XM + 72.) * XM - 96.) * XM + 24.)                        
      GOTO 503                                                           
      END FUNCTION EXPINT                

Subroutine HCol(Atom, NLTE)
! Adapted from MULTI
 Use Param_structure
 Use NLTE_Atom_structure
 Use NLTE_vars
 Implicit None
 Type (NLTE_Atom) :: Atom
 Type (NLTE_variables) :: NLTE
 Real, Dimension(:,:,:), Allocatable :: C
 Integer :: i, j, idepth
 Real :: H1BB, H1BF
 Double Precision T

 Allocate (C(Atom%NK, Atom%NK, NLTE%NDEP))
 C(:,:,:)=0.
 Do idepth=1, NLTE%NDEP
    T=NLTE%Atmo%Temp(idepth)
    Do j=2, Atom%NK-1
       Do i=1, j-1
          C(i,j,idepth)=NLTE%Atmo%ne(idepth)*H1BB(i,j,T)
          C(j,i,idepth)=C(i,j,idepth)*NLTE%NStar(i,idepth)/NLTE%NStar(j,idepth)
       End do
    End do
    Do i=1, Atom%NK-1
       C(i, Atom%NK, idepth)=NLTE%Atmo%ne(idepth)*H1BF(i,T)
       C(Atom%NK, i, idepth)=C(i, Atom%NK, idepth)*NLTE%NStar(i,idepth)/ &
            NLTE%NStar(Atom%NK,idepth)
    End do
 End do
 NLTE%C=C
!
End Subroutine HCol

FUNCTION H1BB (I,J,T)
!
!  DETERMINES H COLLISIONAL RATE COEFFICIENTS UP TO N=50
!
! CONVERSION FACTORS:
!
!      CM**-1 TO EV    HH * CC / EE
!      CM**-1 TO K     HH * CC / BK
!
!      K     TO HZ    BK / HH
!      CM**-1 TO HZ    CC
!
! CONSTANTS:
!
!      EE   ELECTRON CHARGE      1.602189E-12         
!      HH   PLANCK CONSTANT      (ERG S)
!      CC   VELOCITY OF LIGHT    (CM S**-1)
!      EM   ELECTRON REST MASS   (G)
!      UU   ATOMIC MASS UNIT     (G)
!      BK   BOLTZMANN CONSTANT   (ERG K**-1)
!      PI   3.14159265359
!
!
! INPUT:
!
!   I        LOWER LEVEL
!   J        UPPER LEVEL
!   T        TEMPERATURE (K)
!
! OUTPUT:
!
!   H1BB    UPWARDS COLLISIONAL RATE                  I TO J
!
!: H1BB   90-02-28  MODIFICATIONS: (MARTIN J. STIFT)
!:        REWRITTEN.  UP TO 50 BOUND LEVELS + ONE CONTINUUM LEVEL
!:
      INTEGER I,J,K
      DOUBLE PRECISION &
              T,S,X, &
              AI,AMN,AJ,BM,BMN,DELMN,DERG,ENG, &
              DION,FIJ,GAMMN,GMX,G0,G1,G2, &
              EION,EV(50),EE,HH,CC,EM,UU,BK,PI,TEV, &
              B1S2P(9),B1S2S(9),RH1,RH2,H1BB
!
! POLYNOMIAL FITS FOR COLLISIONAL RATE COEFFICIENTS (LYMAN ALPHA)
!
! H(1S) TO H (2P)
!
      DATA B1S2P / -2.814949375869D+1, &
                    1.009828023274D+1, &
                   -4.771961915818D+0, &
                    1.467805963618D+0, &
                   -2.979799374553D-1, &
                    3.861631407174D-2, &
                   -3.051685780771D-3, &
                    1.335472720988D-4, &
                   -2.476088392502D-6/
!
! H(1S) TO H(2S)
!
      DATA B1S2S / -2.833259375256D+1, &
                    9.587356325603D+0, &
                   -4.833579851041D+0, &
                    1.415863373520D+0, &
                   -2.537887918825D-1, &
                    2.800713977946D-2, &
                   -1.871408172571D-3, &
                    6.986668318407D-5, &
                   -1.123758504195D-6/
!
! HYDROGEN MEAN ENERGY LEVELS TAKEN FROM BASHKIN & STONER (1975), PP. 2-3
!
      DATA EV /      0.000, &
                 82259.047, &
                 97492.279, &
                102823.882, &
                105291.646, &
                106632.157, &
                107440.441, &
                107965.049, &
                108324.718, &
                41 * 0.000/
!
! HYDROGEN IONIZATION LEVEL TAKEN FROM BASHKIN & STONER (1975), P. 3
!
      DATA EION / 109678.764/
!
! MISCELLANOUS CONSTANTS
!
      DATA EE / 1.602189D-12/,&
           HH / 6.626176D-27/,&
           CC / 2.99792458D10/,&
           EM / 9.109534D-28/,&
           UU / 1.6605655D-24/,&
           BK / 1.380662D-16/,&
           PI / 3.14159265359/
!
! J MUST BE GREATER THAN I
!
      IF (J.LE.I) then
         print *,'H1BB: ERROR IN ORDER OF ENERGY LEVELS'
         stop
      endif
!
      AI = FLOAT(I)
      AJ = FLOAT(J)
!
! DETERMINE ENERGY LEVELS UP TO N=50
!
      DO 1 K = 10,50
         EV(K) = EION * (1. - 1. / FLOAT(K*K))
    1 CONTINUE
!
! EV, EION, ENG  IN   CM**-1
!
! ENG   ENERGY ASSOCIATED TO TEMPERATURE
!
      ENG = T * BK / HH / CC
!
! ENERGY DIFFERENCE AND IONISATION ENERGY FROM LOWER LEVEL
!
      DERG = EV(J) - EV(I)
      DION = EION  - EV(I)
!
! JOHNSON (1972) FORMULA FOR OSCILLATOR STRENGTH
! SEE JANEV ET AL. (1987), PP. 315-316
!
      X = 1. - (AI/AJ) * (AI/AJ)
!
      IF (I.EQ.1) THEN
         G0 =  1.1330
         G1 = -0.4059
         G2 =  0.07014
      ELSE IF (I.EQ.2) THEN
         G0 =  1.0785
         G1 = -0.2319
         G2 =  0.02947
      ELSE IF (I.GE.3) THEN
         G0 =   0.9935 + (0.2328 - 0.1296 / AI) / AI
         G1 = -(0.6282 - (0.5598 - 0.5299 / AI) / AI ) / AI
         G2 =  (0.3887 - (1.1810 - 1.4700 / AI) / AI ) / (AI * AI)
      ENDIF
!
      GMX = G0 + (G1 + G2 / X) / X
!
! OSCILLATOR STRENGTH - AGREES WITH VALUES IN JANEV ET AL. (1987), P. 315
!
      FIJ = 1.96028052 * GMX * (AI/AJ) / (AJ * AJ * X * X * X)
!
! APN, BP AND BPN OF VRIENS AND SMEETS (1980), EQNS. (11), (13) AND (12)
!
      AMN = 2. * EION * FIJ / DERG
!
      BM = ( 1.4 * LOG(AI) - 0.7 - (0.51 - (1.16 - 0.55 / AI) / &
           AI ) / AI ) / AI
!
      BMN = 4. * (EION/DERG) * (EION/DERG) / (AJ * AJ * AJ) * &
            (1. + (4./3.) * (DION/DERG) + &
             BM * (DION/DERG) * (DION/DERG) )
!
! DELTA_PN AND GAMMA_PN OF VRIENS AND SMEETS (1980), EQNS. (18) AND (19)
!
      S = ABS(AI - AJ)
      DELMN = EXP(-BMN/AMN) + 0.06 * (S/AI) * (S/AI) / AJ
      GAMMN = EION * (3. + 11. * (S/AI)*(S/AI)) * &
              LOG(1. + AI * AI * AI * ENG / EION) / &
              (6. + 1.6 * AJ * S + 0.3 / (S * S) +  &
               0.8 * AJ * SQRT(AJ/S) * ABS(S-0.6))
!
! UPWARD COLLISIONAL RATE  (I TO J) : 
! VRIENS AND SMEETS (1980), EQN. (17), EXCEPT LYMAN ALPHA
! LYMAN ALPHA ACCORDING TO JANEV ET AL. (1987) PP. 18-21 AND 257
!
! TEV IS IN EV IN POLYNOMIAL EXPRESSIONS
!
      IF (I.EQ.1 .AND. J.EQ.2) THEN
!
         TEV = LOG(T * BK / EE)
!
         RH1 = B1S2P(1) + TEV * (B1S2P(2) + TEV * (B1S2P(3) + &
                          TEV * (B1S2P(4) + TEV * (B1S2P(5) + &
                          TEV * (B1S2P(6) + TEV * (B1S2P(7) + &
                          TEV * (B1S2P(8) + TEV *  B1S2P(9) )))))))
         RH2 = B1S2S(1) + TEV * (B1S2S(2) + TEV * (B1S2S(3) + &
                          TEV * (B1S2S(4) + TEV * (B1S2S(5) + &
                          TEV * (B1S2S(6) + TEV * (B1S2S(7) + &
                          TEV * (B1S2S(8) + TEV *  B1S2S(9) )))))))
!
         H1BB = EXP(RH1) + EXP(RH2)
!
      ELSE
!
         H1BB = 1.6D-7 * SQRT (EE / HH / CC) * &
                 (AMN * LOG(0.3D0 * ENG / EION + DELMN) + BMN) * &
                 SQRT(ENG) * EXP(-DERG/ENG) / (ENG + GAMMN) 
!
      ENDIF
!
      RETURN
      END Function H1BB
!
!
      FUNCTION H1BF(I,T)
!
!  DETERMINES H COLLISIONAL IONIZATION COEFFICIENTS UP TO N=50
!
! CONVERSION FACTORS:
!
!      CM**-1 TO EV    HH * CC / EE
!      CM**-1 TO K     HH * CC / BK
!
!      K     TO HZ    BK / HH
!      CM**-1 TO HZ    CC
!
! CONSTANTS:
!
!      EE   ELECTRON CHARGE      1.602189E-12         
!      HH   PLANCK CONSTANT      (ERG S)
!      CC   VELOCITY OF LIGHT    (CM S**-1)
!      EM   ELECTRON REST MASS   (G)
!      UU   ATOMIC MASS UNIT     (G)
!      BK   BOLTZMANN CONSTANT   (ERG K**-1)
!      PI   3.14159265359
!
!
! INPUT:
!
!   I        LOWER LEVEL
!   T        TEMPERATURE (K)
!
! OUTPUT:
!
!   H1BF      COLLISIONAL IONIZATION RATE               I TO K
!
!...................................................................
!
!: H1BF   90-02-28  MODIFICATIONS: (MARTIN J. STIFT)
!:        REWRITTEN.  UP TO 50 BOUND LEVELS + ONE CONTINUUM LEVEL
!:
      INTEGER I,K
      DOUBLE PRECISION &
              T,RATIK,AI,ENG,DION, &
              EION,EV(50),EE,HH,CC,EM,UU,BK,PI,TEV, &
              B1SI(9),B2SI(9),H1BF
!
! H1S TO H+
!
      DATA B1SI /  -3.271396786375D+1, &
                    1.353655609057D+1, &
                   -5.739328757388D+0, &
                    1.563154982022D+0, &
                   -2.877056004391D-1, &
                    3.482559773737D-2, &
                   -2.631976175590D-3, &
                    1.119543953861D-4, &
                   -2.039149852002D-6/
!
! H2S TO H+
!
      DATA B2SI /  -1.973476726029D+1, &
                    3.992702671457D+0, &
                   -1.773436308973D+0, &
                    5.331949621358D-1, &
                   -1.181042453190D-1, &
                    1.763136575032D-2, &
                   -1.616005335321D-3, &
                    8.093908992682D-5, &
                   -1.686664454913D-6/
!
! HYDROGEN MEAN ENERGY LEVELS TAKEN FROM BASHKIN & STONER (1975), PP. 2-3
!
      DATA EV /      0.000,&
                 82259.047,&
                 97492.279,&
                102823.882,&
                105291.646,&
                106632.157,&
                107440.441,&
                107965.049,&
                108324.718,&
                41 * 0.000/
!
! HYDROGEN IONIZATION LEVEL TAKEN FROM BASHKIN & STONER (1975), P. 3
!
      DATA EION / 109678.764/
!
! MISCELLANOUS CONSTANTS
!
      DATA EE / 1.602189E-12/,&
           HH / 6.626176E-27/,&
           CC / 2.99792458E10/,&
           EM / 9.109534E-28/,&
           UU / 1.6605655E-24/,&
           BK / 1.380662E-16/,&
           PI / 3.14159265359/
!
      AI = FLOAT(I)
!
! DETERMINE ENERGY LEVELS UP TO N=50
!
      DO 1 K = 10,50
         EV(K) = EION * (1. - 1. / FLOAT(K*K))
    1 CONTINUE
!
! DION AND ENG  IN   CM**-1
!
! ENG   ENERGY ASSOCIATED TO TEMPERATURE
! DION  IONISATION ENERGY FROM LOWER LEVEL
!
      ENG  = T * BK / HH / CC
      DION = EION  - EV(I)
!
! COLLISIONAL IONIZATION RATE  (I TO K) : VRIENS AND SMEETS (198), EQN. (8)
!
      IF (I.EQ.1) THEN
!
         TEV = LOG(T * BK / EE)
         RATIK = B1SI(1) + TEV * (B1SI(2) + TEV * (B1SI(3) + &
                           TEV * (B1SI(4) + TEV * (B1SI(5) + &
                           TEV * (B1SI(6) + TEV * (B1SI(7) + &
                           TEV * (B1SI(8) + TEV *  B1SI(9) )))))))
         H1BF = EXP(RATIK)
!
      ELSE IF (I.EQ.2) THEN
!
         TEV = LOG(T * BK / EE)
         RATIK = B2SI(1) + TEV * (B2SI(2) + TEV * (B2SI(3) + &
                           TEV * (B2SI(4) + TEV * (B2SI(5) + &
                           TEV * (B2SI(6) + TEV * (B2SI(7) + &
                           TEV * (B2SI(8) + TEV *  B2SI(9) )))))))
         H1BF = EXP(RATIK)
!
      ELSE
!
         H1BF = 9.56D-6 * EXP(-DION/ENG) / ENG / SQRT(ENG) * &
           SQRT(EE / HH / CC) * (EE / HH / CC) / &
          ( (DION/ENG)**2.33 + 4.38*(DION/ENG)**1.72 + 1.32*(DION/ENG) )
!
      ENDIF
!
      RETURN
      END Function H1bf
