      Subroutine ENEQ(NK, NDEP, A, B, NEWMAT)                           
!
! Adapted from MULTI
!                                                                       
!  SOLVES THE EQUATION  SYSTEM  A*X=B.                                  
!  WHEN NEWMAT=TRUE, THE SYSTEM IS REARRANGED INTO U*X=L*B, WHERE U     
!  IS UPPER AND L IS LOWER TRIANGULAR. THESE ARE THEN REUSED IN LATER   
!  CALLS WITH NEWMAT=FALSE AND NEW RIGHT HAND SIDES B. THE SOLUTION     
!  VECTOR IS RETURNED IN B. NO PIVOTING, I.E. THE MATRIX A IS ASSUMED   
!  TO HAVE NONZERO DIAGONAL ELEMENTS.                                   
!                                                                       
!  CODED BY: A. NORDLUND (FEB-1979)                                     
!                                                                       
!  THIS IS A MODIFIED VERSION OF EQSYST WHICH TESTS FOR ZERO ELEMENTS   
!  BELOW THE DIAGONAL AND ALSO STOPS AT THE LAST NON-ZERO ELEMENT ABOVE 
!  THE DIAGONAL. CONSIDERABLE SAVINGS ARE OBTAINED FOR LOOSE MATRICES.  
!                                                                       
!  THIS IS A COLUMN ORIENTED VERSION (M. CARLSSON JAN-1986)             
!  TEMPORARY SCALARS BL, ALL, ALM AND BK ARE USED TO SHOW THE COMPILER  
!  THAT THERE IS NO VECTOR DEPENDENCY IN THE INNERMOST DO-LOOP          
!                                                                       
!: ENEQ   90-06-08  NEW ROUTINE: (MARTIN J STIFT)                       
!:        LIKE EQSYST BUT FOR A BLOCK DIAGONAL GRAND MATRIX             
!:                                                                      
        Implicit None
        Real, Dimension(NK, NK, NDEP) :: A
        Real, Dimension(NK, NDEP) :: B
        Integer, Dimension(:,:), Allocatable, Save :: LASTN
        Logical :: NEWMAT
        Logical, SAVE :: Warning
        Integer :: NK, NDEP, N, MD, L, K, M
        Real :: ALL, BL, ALM, BK
!                                               
        If (.not. Allocated(LASTN)) Allocate(LASTN(NK,NDEP))
!                                
        N = NK                                                             
!       
        Do MD=1, NDEP
!                                                                       
         If (NEWMAT) then
!                                                                       
!     FIND THE LAST NON-ZERO ELEMENT IN EACH COLUMN                     
!                                                                       
            Do L=1, N                                               
               Do K=N, L+1, -1                                   
                  If (A(K, L, MD) .ne. 0.0) GOTO 20                      
               End do
               K=L                                                     
   20          LASTN(L, MD)=K                                         
            End do
!                                                                       
!  COLUMN LOOP: ELIMINATE ELEMENTS BELOW THE DIAGONAL IN COLUMN L.      
!                                                                       
            Do L=1, N - 1                                           
!                                                                       
!  STORE -A(K,L)/A(L,L) IN ELEMENT A(K,L)                               
!  MULTIPLY RIGHT HAND SIDE WITH -A(K,L)/A(L,L)                         
!                                                                       
               ALL = A (L, L, MD)  
               If (ALL .eq. 0) then
                  If (.not. Warning) Then
                     Print *,'Warning in ENEQ, diagonal element of A is zero'
                     Print *,'(trying to recover...)'
!                     Warning=.True.
                  End if
                  ALL=1.
                  A(L, L, MD)=ALL
               End if
                                      
               BL = B (L, MD)                                            
               Do K = L + 1, LASTN (L, MD)                            
                  A (K, L, MD) = - A (K, L, MD) / ALL                    
                  B (K, MD) = B (K, MD) + A (K, L, MD) * BL              
               End do
!                                                                       
!  ADD FRACTION -A(K,L)/A(L,L) OF ROW L TO ROW K.                       
!  IN EACH COLUMN GO THROUGH ALL ROWS                                   
!                                                                       
               Do M = L + 1, N                                        
                  If (A (L, M, MD) .ne. 0.0) then
                     ALM = A (L, M, MD)                                  
                     LASTN (M, MD)=MAX(LASTN (L, MD) , LASTN (M, MD))
                     Do K = L + 1, LASTN (L, MD)                      
                        A (K, M, MD) = A (K, M, MD) + A (K, L, MD)      &
                        * ALM                                           
                     End do
                  End if
               End do
!                                                                       
            End do ! Do in L
!                                                                       
         Else ! if .not. NEWMAT
!                                                                       
            Do L = 1, N - 1                                           
               BL = B (L, MD)                                            
               Do K = L + 1, LASTN (L, MD)                            
                  B (K, MD) = B (K, MD) + A (K, L, MD) * BL              
               End do
            End do
!                                                                       
         End if ! End If NEWMAT
!                                                                       
! BACKSUBSTITUTE                                                        
!                                                                       
         Do K = N, 1, - 1                                            
            BK = B (K, MD)                                               
            Do L = K + 1, N                                          
               BK = BK - A (K, L, MD) * B (L, MD)                        
            End do
            B (K, MD) = BK / A (K, K, MD)                                
         End do
!                                                                       
      End do ! Do in MD
!                                                                       
      Return
    End Subroutine ENEQ
