! This routine calls a suitable formal solution routine to compute the
! emergent Stokes vector.
! If formal_solut == 0 , it is chosen automatically
!                    == 1 , it uses Hermite with parabolic interpolation
!                    == 2 , it uses WPM with parabolic interpolation
!
! By Hermite I mean the Hermitian solution method of Bellot Rubio, Ruiz
!   Cobo and Collados 1998, ApJ, 506, 805
! By WPM I mean the Weakly Polarizing Media approximation of Sanchez
!   Almeida and Trujillo Bueno 1999, ApJ, in press.
!
Subroutine Formal_solution(npoints, formal_solut, ltau_500, Absorp, &
     Source_f, Stokes, ichoice)
  Implicit None
  Real, Dimension (npoints) :: ltau_500, Source_f
  Real, Dimension (4, 4, npoints) :: Absorp
  Real, Dimension (4) :: Stokes
  Integer :: formal_choice, ipoint, npoints, formal_solut
  Logical :: suitable
  Real, Parameter :: Threshold = 10.
  Integer :: ichoice
!
  formal_choice=formal_solut
  If (formal_choice .eq. 0) then ! Choose one
!    See if WPM is suitable for this case
     suitable=.TRUE.
     ipoint=1
     Do while (ipoint .le. npoints .and. suitable)
        suitable = Absorp(1,1,ipoint) .gt. Threshold*( &
             Sum(Abs(Absorp(1,2:4,ipoint))))
        ipoint=ipoint+1
     End Do
     If (suitable) then ! WPM
        formal_choice=2 !  WPM
     else ! Hermite
        formal_choice=1 !  Hermite
     End if
  End if
!
  If (formal_choice .eq. 1) then
     Call Hermite(npoints, ltau_500, Absorp, Source_f, Stokes)
     ichoice=1
  Else if (formal_choice .eq. 2) then
     Call WPM(npoints, ltau_500, Absorp, Source_f, Stokes)
     ichoice=2
  End if
End Subroutine Formal_solution
