! Parameters for the multinest algorithm
module multinest_mod
use global_vars_mod
use maths_mod
use Nested
Use Param_structure
Use Model_structure
Use Line_data_structure
implicit none

!dimensionality
	integer :: sdim
      
      
!priors on the parameters
!uniform priors (-6,6) are used for all dimensions & are set in main.f90
   real(kind=8), allocatable :: prior_range(:,:) !spriorran(sdim,2)

   integer, allocatable :: nest_pWrap(:) ! Wraparound parameters
      
! Parameters for Nested Sampler
	
!whether to do multimodal sampling
	logical :: nest_mmodal
 	parameter(nest_mmodal=.true.)
	
!max no. of live points
!    integer :: nest_nlive
! 	parameter(nest_nlive=300)
      
!tot no. of parameters, should be sdim in most cases but if you need to
!store some additional parameters with the actual parameters then
!you need to pass them through the likelihood routine
	integer :: nest_nPar

	logical :: nest_ceff
	parameter(nest_ceff = .TRUE.)
      
!seed for nested sampler, -ve means take it from sys clock
	integer :: nest_rseed
	parameter(nest_rseed=-1)
      
! !evidence tolerance factor
!    real(kind=8) :: nest_tol
!    parameter(nest_tol=0.1)
!       
! !enlargement factor reduction parameter
! 	real(kind=8) :: nest_efr
!    parameter(nest_efr=0.3d0)
      
!root for saving posterior files
	character*100 nest_root	
	
!no. of iterations after which the ouput files should be updated
	integer nest_updInt
	parameter(nest_updInt=100)

!null evidence (set it to very high negative no. if null evidence is unknown)
	real*8 nest_Ztol
	parameter(nest_Ztol=-1.d90)
      
! !max modes expected, for memory allocation
!   	integer nest_maxModes
!   	parameter(nest_maxModes=20)
      
!no. of parameters to cluster (for mode detection)
  	integer nest_nClsPar  	
      
!whether to resume from a previous run
  	logical nest_resume
  	parameter(nest_resume=.false.)
      
!feedback on the sampling progress?
  	logical nest_fb
  	parameter(nest_fb=.true.)

	real(kind=8) :: maxlhood

contains

!-------------------------------------------------------------------
! Calculate posterior distribution
!-------------------------------------------------------------------
	subroutine synthesize_profiles(last, output)
	real(kind=8) :: last(:), output(:,:)
	integer :: i, j
		
		call set_modified_atmospheres(last)
		
      Stokes_Tot = 0.d0
		do i = 1, atmos%ncomponents
			Stokes_Syn = 0.d0
			atmos%perturbed(i)%v_mac = 1.d5 * last(chain%nparam+1)
 			Call Forward(Params, Line, Region, atmos%perturbed(i), Stokes_Syn, Hydro, Error)
			Stokes_Tot = Stokes_Tot + atmos%ff(i) * Stokes_Syn
		enddo

! Spectroscopy
		if (observation_mode == 0) then
			do i = 1, obs%npoints
				do j = 1, 4
					output(j,i) = Stokes_Tot(4*(i-1)+j)				
				enddo			
			enddo
		else
! Filters
			do j = 1, 4

! Put the Stokes parameter j into a vector. We reuse Stokes_Syn
				do i = 1, filter%nlambda
					Stokes_Syn(i) = Stokes_Tot(4*(i-1)+j)					
				enddo

! And then apply the filters
				do i = 1, obs%npoints
 					output(j,i) = &
						int_tabulated(filter%lambda, filter%transmission(i,:)*Stokes_Syn(1:filter%nlambda))
				enddo

			enddo
		endif

	end subroutine synthesize_profiles

!-------------------------------------------------------------------
! Calculate prior distribution
!-------------------------------------------------------------------
	function calculate_prior(last)
	real(kind=8) :: last(:), calculate_prior	
	integer :: i, j, k, loop

		calculate_prior = 1.0

		loop = 1
		do j = 1, n_param_per_component
			do i = 1, atmos%ncomponents
				do k = 1, atmos%n_nodes(i,j)

! Dirac prior
					if (atmos%priors(i)%typ(j) == 2) then
						last(loop) = atmos%priors(i)%mu(j)
					endif

! Gaussian prior
					if (atmos%priors(i)%typ(j) == 1) then
						calculate_prior = calculate_prior * &
							exp(-(last(loop)-atmos%priors(i)%mu(j))**2 / (2.0*atmos%priors(i)%sigma(j)**2)) / &
							(sqrt(2.d0*3.14159265359d0)*atmos%priors(i)%sigma(j))
					endif

! Uniform prior
					if (atmos%priors(i)%typ(j) == 0) then
						calculate_prior = calculate_prior / (atmos%priors(i)%maxim(j)-atmos%priors(i)%minim(j))
					endif
					
					loop = loop + 1
				enddo
			enddo
		enddo
		
	end function calculate_prior

	
!-------------------------------------------------------------------
! Calculate posterior distribution
!-------------------------------------------------------------------
	function calculate_posterior(last, observations, chi2)
	real(kind=8) :: last(:), calculate_posterior
	type(prior_type) :: prior
	type(observations_type) :: observations
	real(kind=8) :: P_prior, logP_likelihood, logP_posterior, chi2, normalization_loglike

! Calculate prior distribution
		P_prior = calculate_prior(last)
		
		if (P_prior == 0.0) then
			calculate_posterior = 0.0
			return
		else
			
! Evaluate the direct problem
 			call synthesize_profiles(last, observations%model_eval)

! Calculate the chi^2
			chi2 = sum((observations%model_eval-observations%obs_y)**2 / &
				(observations%obs_sigma**2))
							
! Normalization factors
			normalization_loglike = 0.5d0 * 4.d0 * observations%npoints * log(2.d0*3.14159265359d0) + sum(log(observations%obs_sigma))
			
! Calculate likelihood and posterior
			logP_likelihood = -normalization_loglike - 0.5 * chi2

! Add 
			logP_posterior = logP_likelihood + log(P_prior)

		endif

		calculate_posterior = logP_posterior

	end function calculate_posterior

!----------------------------------------------------------------------
	subroutine nest_Sample
   integer :: nclusters,context !total number of clusters found
   integer :: maxNode !variables used by the posterior routine
   
   	call nestRun(nest_mmodal,nest_ceff,nest_nlive,nest_tol,nest_efr,sdim,nest_nPar, &
   		nest_nClsPar,nest_maxModes,nest_updInt,nest_Ztol,nest_root,nest_rseed, nest_pWrap, &
   		nest_fb,nest_resume,getLogLike,context)

	end subroutine nest_Sample
!----------------------------------------------------------------------

!----------------------------------------------------------------------
! Wrapper around Likelihood Function
! Cube(1:n_dim) has nonphysical parameters
! scale Cube(1:n_dim) & return the scaled parameters in Cube(1:n_dim) &
! additional parameters in Cube(n_dim+1:nPar)
! return the log-likelihood in lnew
!----------------------------------------------------------------------
	subroutine getLogLike(Cube,n_dim,nPar,lnew,context)

   integer :: n_dim,nPar,context
   real(kind=8) :: lnew,Cube(nPar)
   
   !call your loglike function here   
   	call slikelihood(Cube,lnew)

	end subroutine getLogLike

!----------------------------------------------------------------------
! Evaluate likelihood
!----------------------------------------------------------------------
	subroutine slikelihood(Cube,slhood)              
	real(kind=8) :: Cube(nest_nPar),slhood
	real(kind=8) :: temp(sdim),dist,loclik, chi2, ff_total
	integer :: i,j
	logical :: physical
	

		Cube(1:sdim) = (prior_range(1:sdim,2)-prior_range(1:sdim,1))*Cube(1:sdim) + prior_range(1:sdim,1)


! If more than one component, the last filling factor is not a free
! parameter but fulfills ff(last) = 1-sum(ff(1:last-1))
 		if (atmos%ncomponents > 1) then
 
! Calculate the value of the last filling factor
! and put it into Cube(nest_npar)
			ff_total = 0.d0
         do i = chain%nparam_not_ffs+1, chain%nparam
         	ff_total = ff_total + Cube(i)
			enddo

			if (ff_total /= 1.d0) then
         	Cube(chain%nparam) = 1.d0 - ff_total + Cube(chain%nparam)
         endif
      endif
			
		chain%ncalls = chain%ncalls + 1
 		slhood = calculate_posterior(Cube, obs, chi2)

		if (slhood > maxlhood) then
			maxlhood = slhood
			chain%best_likelihood = slhood
			chain%best_chi2 = chi2
			chain%best_parameters = Cube
		endif
		
		if (int(chain%ncalls / 1000) == chain%ncalls / 1000.) then
			write(*,*) '---------------------------------'
			write(*,*) 'Best likelihood = ', chain%best_likelihood
			write(*,*) 'Best reduced chi2 = ', chain%best_chi2 / (4.d0*obs%npoints)
			write(*,*) '---------------------------------'			
		endif

		return
			
	end subroutine slikelihood


!----------------------------------------------------------------------
! Do MULTINEST MCMC
!----------------------------------------------------------------------
	subroutine do_multinest(chain, prior, initialize)
	type(markov_chain_type) :: chain
	type(prior_type) :: prior
	type(observations_type) :: observations	
	integer :: step_from, step_to
	logical :: initialize
	
	real(kind=8) :: chi2
	logical :: physical
	integer :: i, loop, j

! This includes the macroscopic velocity
		nest_nPar = chain%nparam + 1
		sdim = nest_npar
		chain%best_likelihood = -1.d100

! If there are several components, one of the filling factors is not a
! free parameter but is related to the sum of the others
! 		if (atmos%ncomponents > 1) then
!       	sdim = nest_npar - 1
!       endif
      
		nest_nClsPar = sdim

		nest_root = chain%filename

		maxlhood = -1.d100

! Allocate memory for the prior ranges
		allocate(prior_range(nest_nPar,2))

! For periodic boundary conditions
		allocate(nest_pWrap(sdim))
		nest_pWrap = 0

! Set the prior ranges that have been read before
 		loop = 1
 		do i = 1, n_param_per_component
 			do j = 1, atmos%ncomponents
 				prior_range(loop:loop+atmos%n_nodes(j,i)-1,1) = atmos%priors(j)%minim(i)
 				prior_range(loop:loop+atmos%n_nodes(j,i)-1,2) = atmos%priors(j)%maxim(i)
 				loop = loop + atmos%n_nodes(j,i)
 			enddo
 		enddo
 		
! Prior range for vmac
 		prior_range(nest_nPar,1) = atmos%priors(1)%minim(n_param_per_component+1)
 		prior_range(nest_nPar,2) = atmos%priors(1)%maxim(n_param_per_component+1)
 		
 		if (verbose_level > 1) then
			write(*,*) 'Number of atmospheric parameters = ', chain%nparam
			write(*,*) 'Number of free parameters (including vmac) = ', sdim
		endif

! Do the actual sampling
		call nest_Sample

		deallocate(prior_range)		
		
	end subroutine do_multinest

end module multinest_mod