#!/usr/bin/env python
# Convert string to lower case up to the first occurrence of a separator
def lower_to_sep(string, separator='='):
	line=string.partition(separator)
	string=str(line[0]).lower()+str(line[1])+str(line[2])
	return string

from configobj import ConfigObj
import sys
import os
from subprocess import call

if (len(sys.argv) != 2):
	print "You need to give the configuration file"
	exit()

print "Using configuration file = "+sys.argv[1]

# Transform all keys to lowercase to avoid problems with
# upper/lower case
f = open(sys.argv[1],'r')
input_lines = f.readlines()
f.close()
input_lower = ['']
for l in input_lines:
	input_lower.append(lower_to_sep(l)) # Convert keys to lowercase

config = ConfigObj(input_lower)

prior_dict = {"U": '0', "G": '1', "D": '2'}

file = open('conf.input','w')

# Write general information
file.write(config['general']['number of lines']+'\n')
file.write(config['general']['number of regions']+'\n')
file.write(config['general']['number of components']+'\n')
file.write(config['general']['cosinus heliocentric angle']+'\n')
file.write("'"+config['general']['file with observations']+"'\n")
file.write("'"+config['general']['file with output results']+"'\n")
if (config['general']['observing mode (spectro/filter)'] == 'SPECTRO'):
	file.write("0\n")
	file.write("'"+config['general']['file with transmission filters']+"'\n")
if (config['general']['observing mode (spectro/filter)'] == 'FILTER'):
	file.write("1\n")
	file.write("'"+config['general']['file with transmission filters']+"'\n")

file.write(config['multinest']['evidence tolerance']+'\n')
file.write(config['multinest']['enlargement factor reduction']+'\n')
file.write(config['multinest']['maximum number of modes']+'\n')
file.write(config['multinest']['number of live points']+'\n')

n_models = int(config['general']['number of components'])
n_regions = int(config['general']['number of regions'])
n_lines = int(config['general']['number of lines'])

# write information for the models
for i in range(n_models):
	file.write("'"+config['atmosphere '+str(i+1)]['initial file']+"'\n")
	file.write(config['atmosphere '+str(i+1)]['nodes']['temperature']+' '+
		config['atmosphere '+str(i+1)]['nodes']['microturbulent velocity']+' '+
		config['atmosphere '+str(i+1)]['nodes']['magnetic field']+' '+
		config['atmosphere '+str(i+1)]['nodes']['inclination']+' '+
		config['atmosphere '+str(i+1)]['nodes']['azimuth']+' '+
		config['atmosphere '+str(i+1)]['nodes']['los velocity']+' '+
		config['atmosphere '+str(i+1)]['nodes']['filling factor']+'\n')
	file.write("t_"+str(i+1)+"\n")
	file.write(prior_dict[config['atmosphere '+str(i+1)]['priors']['temperature']['type']]+' '+
		config['atmosphere '+str(i+1)]['priors']['temperature']['min']+' '+
		config['atmosphere '+str(i+1)]['priors']['temperature']['max']+' '+
		config['atmosphere '+str(i+1)]['priors']['temperature']['center']+' '+
		config['atmosphere '+str(i+1)]['priors']['temperature']['sigma']+'\n')
	file.write("vmic_"+str(i+1)+"\n")
	file.write(prior_dict[config['atmosphere '+str(i+1)]['priors']['microturbulent velocity']['type']]+' '+
		config['atmosphere '+str(i+1)]['priors']['microturbulent velocity']['min']+' '+
		config['atmosphere '+str(i+1)]['priors']['microturbulent velocity']['max']+' '+
		config['atmosphere '+str(i+1)]['priors']['microturbulent velocity']['center']+' '+		
		config['atmosphere '+str(i+1)]['priors']['microturbulent velocity']['sigma']+'\n')
	file.write("b_"+str(i+1)+"\n")
	file.write(prior_dict[config['atmosphere '+str(i+1)]['priors']['magnetic field']['type']]+' '+
		config['atmosphere '+str(i+1)]['priors']['magnetic field']['min']+' '+
		config['atmosphere '+str(i+1)]['priors']['magnetic field']['max']+' '+
		config['atmosphere '+str(i+1)]['priors']['magnetic field']['center']+' '+
		config['atmosphere '+str(i+1)]['priors']['magnetic field']['sigma']+'\n')
	file.write("in_"+str(i+1)+"\n")
	file.write(prior_dict[config['atmosphere '+str(i+1)]['priors']['inclination']['type']]+' '+
		config['atmosphere '+str(i+1)]['priors']['inclination']['min']+' '+
		config['atmosphere '+str(i+1)]['priors']['inclination']['max']+' '+
		config['atmosphere '+str(i+1)]['priors']['inclination']['center']+' '+
		config['atmosphere '+str(i+1)]['priors']['inclination']['sigma']+'\n')
	file.write("az_"+str(i+1)+"\n")
	file.write(prior_dict[config['atmosphere '+str(i+1)]['priors']['azimuth']['type']]+' '+
		config['atmosphere '+str(i+1)]['priors']['azimuth']['min']+' '+
		config['atmosphere '+str(i+1)]['priors']['azimuth']['max']+' '+
		config['atmosphere '+str(i+1)]['priors']['azimuth']['center']+' '+
		config['atmosphere '+str(i+1)]['priors']['azimuth']['sigma']+'\n')
	file.write("vlos_"+str(i+1)+"\n")
	file.write(prior_dict[config['atmosphere '+str(i+1)]['priors']['los velocity']['type']]+' '+
		config['atmosphere '+str(i+1)]['priors']['los velocity']['min']+' '+
		config['atmosphere '+str(i+1)]['priors']['los velocity']['max']+' '+
		config['atmosphere '+str(i+1)]['priors']['los velocity']['center']+' '+
		config['atmosphere '+str(i+1)]['priors']['los velocity']['sigma']+'\n')
	file.write("ff_"+str(i+1)+"\n")
	file.write(prior_dict[config['atmosphere '+str(i+1)]['priors']['filling factor']['type']]+' '+
		config['atmosphere '+str(i+1)]['priors']['filling factor']['min']+' '+
		config['atmosphere '+str(i+1)]['priors']['filling factor']['max']+' '+
		config['atmosphere '+str(i+1)]['priors']['filling factor']['center']+' '+
		config['atmosphere '+str(i+1)]['priors']['filling factor']['sigma']+'\n')

# Prior for vmacroscopic
file.write("vmac\n")
file.write(prior_dict[config['macroscopic velocity prior']['type']]+' '+
	config['macroscopic velocity prior']['min']+' '+
	config['macroscopic velocity prior']['max']+' '+
	config['macroscopic velocity prior']['center']+' '+
	config['macroscopic velocity prior']['sigma']+'\n')
		
for i in range(n_regions):
	file.write(config['region '+str(i+1)]['number of wavelengths']+' '+
		config['region '+str(i+1)]['first wavelength']+' '+
		config['region '+str(i+1)]['wavelength step']+' '+
		config['region '+str(i+1)]['macro enhancement']+' '+
		config['region '+str(i+1)]['layer']+'\n')

for i  in range(n_lines):
	file.write(config['line '+str(i+1)]['central wavelength']+' '+
		config['line '+str(i+1)]['energy lower']+ ' '+
		config['line '+str(i+1)]['vdw enhancement']+ ' '+
		config['line '+str(i+1)]['loggf']+ ' '+
		config['line '+str(i+1)]['atomic number']+ ' '+
		config['line '+str(i+1)]['ionization stage']+ ' '+
		config['line '+str(i+1)]['atomic weight']+ ' '+
		config['line '+str(i+1)]['sigma0']+' '+
		config['line '+str(i+1)]['alpha']+'\n')
	file.write(config['line '+str(i+1)]['lower level']['multiplicity']+' '+
		"'"+config['line '+str(i+1)]['lower level']['term']+"' "+
		config['line '+str(i+1)]['lower level']['j']+'\n')
	file.write(config['line '+str(i+1)]['upper level']['multiplicity']+' '+
		"'"+config['line '+str(i+1)]['upper level']['term']+"' "+
		config['line '+str(i+1)]['upper level']['j']+'\n')
	
file.close()

# Run the code
try:
	call(['./bayeslte', 'conf.input'])
	#os.remove('conf.input')
except:
	#os.remove('conf.input')
	call(['pkill', '-9', 'bayeslte'])
